#include "MaterialManager.h"
#include "ViscoplasticSettingWidget.h"

#include <QToolButton>
#include <QTreeWidgetItem>
#include <quuid.h>
#include <utils/ServiceRegistry.h>

MaterialManager::MaterialManager()
{
    m_projectManagerService = ServiceRegistry<ProjectManagerService>::instance().getService();
    m_uiFrameService = ServiceRegistry<UiFrameService>::instance().getService();
    setupTreeWidget();

}

void MaterialManager::setupTreeWidget()
{
    //--------------材料
    QToolButton *addMetrialButton = new QToolButton;
    addMetrialButton->setText(QStringLiteral("添加材料"));
    addMetrialButton->setIcon(QIcon(":/resources/menu/addMetrial.svg"));
    m_uiFrameService->addMenuButton(QStringLiteral("材料"), "material", addMetrialButton);



    m_materialManagerWidget =  new MaterialManagerWidget(this);
    StrengthSettingWidget *m_strengthSettingWidget = new StrengthSettingWidget();
    PhysicalParametersSettingWidget *m_physicalParametersSettingWidget = new PhysicalParametersSettingWidget();
    HeterogeneousSettingWidget *m_heterogeneousSettingWidget = new HeterogeneousSettingWidget();
    ConstitutiveModelSettingWidget *m_constitutiveModelSettingWidget = new ConstitutiveModelSettingWidget();

    QString uuid =  QUuid::createUuid().toString();
    m_rootItem = new QTreeWidgetItem();
    m_rootItem->setText(0, QStringLiteral("材料"));
    m_rootItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->addTreeItem(m_rootItem);
    m_projectManagerService->setItemWidget(uuid, m_materialManagerWidget);

    connect(addMetrialButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(m_rootItem);
    });

    QTreeWidgetItem *childItem = nullptr;
}

void MaterialManager::addMaterial(QString name,QString uuid)
{
    StrengthSettingWidget *strengthSettingWidget = new StrengthSettingWidget();
    PhysicalParametersSettingWidget *physicalParametersSettingWidget = new PhysicalParametersSettingWidget();
    HeterogeneousSettingWidget *heterogeneousSettingWidget = new HeterogeneousSettingWidget();
    ConstitutiveModelSettingWidget *constitutiveModelSettingWidget = new ConstitutiveModelSettingWidget();
    ViscoplasticSettingWidget *viscoplasticSettingWidget = new ViscoplasticSettingWidget();


    QTreeWidgetItem *rootItem = new QTreeWidgetItem(m_rootItem);
    rootItem->setText(0, name);
    rootItem->setData(0, Qt::UserRole, uuid);

    //    m_itemMap[QStringLiteral("强度")] = new StrengthSettingWidget;
    //    m_itemMap[QStringLiteral("变形和物理参数")] = new PhysicalParametersSettingWidget;
    //    m_itemMap[QStringLiteral("非均质选型")] = new HeterogeneousSettingWidget;
    //    m_itemMap[QStringLiteral("本构模型")] = new ConstitutiveModelSettingWidget;

    QTreeWidgetItem *childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("强度"));
    childItem->setData(0, Qt::UserRole, uuid + "_strength");
    m_projectManagerService->setItemWidget(uuid + "_strength", strengthSettingWidget);

    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("变形和物理参数"));
    childItem->setData(0, Qt::UserRole, uuid + "_phyical");
    m_projectManagerService->setItemWidget(uuid + "_phyical", physicalParametersSettingWidget);

    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("非均质选型"));
    childItem->setData(0, Qt::UserRole, uuid + "_heterogeneous");
    m_projectManagerService->setItemWidget(uuid + "_heterogeneous", heterogeneousSettingWidget);

    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("本构模型"));
    childItem->setData(0, Qt::UserRole, uuid + "_constitutive");
    m_projectManagerService->setItemWidget(uuid + "_constitutive", constitutiveModelSettingWidget);

    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("粘塑性参数"));
    childItem->setData(0, Qt::UserRole, uuid + "_visco");
    m_projectManagerService->setItemWidget(uuid + "_visco", viscoplasticSettingWidget);

    m_rootItem->setExpanded(true);
}

void MaterialManager::removeAllMaterial()
{
     m_rootItem->takeChildren();
}
