#include "MaterialManager.h"
#include "MaterialManagerWidget.h"
#include "ui_MaterialManagerWidget.h"

#include <QCheckBox>
#include <QDebug>
#include <QUuid>

MaterialManagerWidget::MaterialManagerWidget(MaterialManager *manager,QWidget *parent) :
    m_materialManager(manager),
    QWidget(parent),
    ui(new Ui::MaterialManagerWidget)
{
    ui->setupUi(this);
    ui->tableWidget->verticalHeader()->setVisible(false); // 隐藏行号
    // 设置表头不可选中
    ui->tableWidget->horizontalHeader()->setSectionsClickable(false);
    ui->tableWidget->horizontalHeader()->setStyleSheet("QHeaderView::section {"
                                                       "background-color: rgb(240, 240, 240);"
                                                       "font-size: 17px;"
                                                       "border: 1px solid rgb(214,216,218);"  // 设置边框为黑色实线
                                                       "}"
                                                       "QHeaderView::section:selected {"
                                                       "background-color: lightblue;"  // 选中时的背景色
                                                       "}");

    ui->tableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    ui->tableWidget->horizontalHeader()->setSectionResizeMode(0,QHeaderView::ResizeToContents);
    //    ui->tableWidget->horizontalHeader()->setSectionResizeMode(1,QHeaderView::ResizeToContents);
    ui->tableWidget->horizontalHeader()->setSectionResizeMode(2,QHeaderView::ResizeToContents);
    ui->tableWidget->horizontalHeader()->setSectionResizeMode(3,QHeaderView::ResizeToContents);

    QStringList names;
    names << QStringLiteral("材料1") << QStringLiteral("材料2") << QStringLiteral("材料3") << QStringLiteral("材料4"); // 示例数据，可以根据实际情况替换
    int row = ui->tableWidget->rowCount(); // 获取当前行数，也可以使用固定的行数

    // 在末尾插入新行
    // 假设有三行数据需要插入
    for (int row = 0; row < 4; ++row) {
        // 序号
        ui->tableWidget->insertRow(row);
        QTableWidgetItem *numberItem = new QTableWidgetItem(QString::number(row + 1));
        numberItem->setTextAlignment(Qt::AlignCenter);
        numberItem->setBackground(QColor(245, 245, 245));
        numberItem->setFlags(numberItem->flags() & ~Qt::ItemIsEditable);
        ui->tableWidget->setItem(row, 0, numberItem);

        // 名称（假设名称存储在一个QStringList中）
        QTableWidgetItem *nameItem = new QTableWidgetItem(names[row]);
        ui->tableWidget->setItem(row, 1, nameItem);
        nameItem->setTextAlignment(Qt::AlignCenter);
        nameItem->setBackground(QColor(245, 245, 245));

        // 复选框
        QCheckBox *checkBox = new QCheckBox();
        QWidget *checkBoxWidget = new QWidget();
        QHBoxLayout *layoutCheckBox = new QHBoxLayout(checkBoxWidget);
        layoutCheckBox->addWidget(checkBox);
        layoutCheckBox->setAlignment(Qt::AlignCenter);
        layoutCheckBox->setContentsMargins(0, 0, 0, 0);
        checkBoxWidget->setLayout(layoutCheckBox);
        ui->tableWidget->setCellWidget(row, 2, checkBoxWidget);

        // 删除按钮
        QPushButton *deleteButton = new QPushButton(QStringLiteral("删除"));
        deleteButton->setFixedWidth(50);
        connect(deleteButton, &QPushButton::clicked, [=]() {
            // 获取按钮所在的行
            QModelIndex index = ui->tableWidget->indexAt(deleteButton->mapToParent(QPoint(0, 0)));
            int rowIndex = index.row();

            // 处理删除按钮点击事件
            ui->tableWidget->removeRow(rowIndex);

            // 更新序号
            for (int row = rowIndex; row < ui->tableWidget->rowCount(); ++row) {
                QTableWidgetItem *numberItem = ui->tableWidget->item(row, 0);
                if (numberItem) {
                    numberItem->setText(QString::number(row + 1));
                }
            }
        });
        ui->tableWidget->setCellWidget(row, 3, deleteButton);
    }
}


MaterialManagerWidget::~MaterialManagerWidget()
{
    delete ui;
}

void MaterialManagerWidget::on_pushButton_clicked()
{
    m_materialManager->removeAllMaterial();
    for (int row = 0; row < ui->tableWidget->rowCount(); ++row) {
        QWidget *checkBoxWidget = ui->tableWidget->cellWidget(row, 2);
        if (checkBoxWidget) {
            QCheckBox *checkBox = checkBoxWidget->findChild<QCheckBox *>();
            if (checkBox && checkBox->isChecked()) {
                QTableWidgetItem *nameItem = ui->tableWidget->item(row, 1);
                if (nameItem) {
                    QString name = nameItem->text();
                    qDebug() << "Row" << row + 1 << "is checked. Name:" << name;
                    //添加材料 之后改为uuid
                    //-----------------------------------------------------------------
                    m_materialManager->addMaterial(name,QUuid::createUuid().toString());
                }
            }
        }
    }
}

void MaterialManagerWidget::on_pushButton_3_clicked()
{
    int rowCount = ui->tableWidget->rowCount();

        // 在表格中插入新行
        ui->tableWidget->insertRow(rowCount);

        // 可以在新行中添加数据，比如序号、名称等
        QTableWidgetItem *numberItem = new QTableWidgetItem(QString::number(rowCount + 1));
        ui->tableWidget->setItem(rowCount, 0, numberItem);
        numberItem->setTextAlignment(Qt::AlignCenter);
        numberItem->setBackground(QColor(245, 245, 245));
        numberItem->setFlags(numberItem->flags() & ~Qt::ItemIsEditable);

        // 假设 names 是一个 QStringList 存储名称
        QTableWidgetItem *nameItem = new QTableWidgetItem(QStringLiteral("材料")+QString::number(rowCount + 1)); // 替换为你的实际数据来源
        ui->tableWidget->setItem(rowCount, 1, nameItem);
        nameItem->setTextAlignment(Qt::AlignCenter);
        nameItem->setBackground(QColor(245, 245, 245));

        // 其他列的数据项添加，根据需要自行添加代码
        // 复选框
        QCheckBox *checkBox = new QCheckBox();
        QWidget *checkBoxWidget = new QWidget();
        QHBoxLayout *layoutCheckBox = new QHBoxLayout(checkBoxWidget);
        layoutCheckBox->addWidget(checkBox);
        layoutCheckBox->setAlignment(Qt::AlignCenter);
        layoutCheckBox->setContentsMargins(0, 0, 0, 0);
        checkBoxWidget->setLayout(layoutCheckBox);
        ui->tableWidget->setCellWidget(rowCount, 2, checkBoxWidget);

        // 如果需要添加按钮或者其他控件，可以参考之前的示例
        QPushButton *deleteButton = new QPushButton(QStringLiteral("删除"));
        deleteButton->setFixedWidth(50);
        connect(deleteButton, &QPushButton::clicked, [=]() {
            // 删除按钮对应的行处理逻辑
            QModelIndex index = ui->tableWidget->indexAt(deleteButton->mapToParent(QPoint(0, 0)));
            int rowIndex = index.row();

            // 处理删除按钮点击事件
            ui->tableWidget->removeRow(rowIndex);

            // 更新序号
            for (int row = 0; row < ui->tableWidget->rowCount(); ++row) {
                QTableWidgetItem *numberItem = ui->tableWidget->item(row, 0);
                if (numberItem) {
                    numberItem->setText(QString::number(row + 1));
                }
            }
        });
        ui->tableWidget->setCellWidget(rowCount, 3, deleteButton);
}
