#include "DisplacementLimitSettingWidget.h"
#include "ui_DisplacementLimitSettingWidget.h"
#include "PhysicalField.h"
#include <qcombobox.h>
//#include <Warning.h>
#include <QMessageBox>

DisplacementLimitSettingWidget::DisplacementLimitSettingWidget(PhysicalField *physicalField,QWidget *parent) :
    QWidget(parent),
    m_physicalField(physicalField),
    ui(new Ui::DisplacementLimitSettingWidget)
{
    ui->setupUi(this);
    m_boundarySetting = m_physicalField->getBoundarySetting();
    m_visualWidgetService = ServiceRegistry<VisualWidgetService>::instance().getService();
    m_preProcessingService = ServiceRegistry<PreProcessingService>::instance().getService();

    //显示模式
    connect(ui->allShowRadio, &QRadioButton::toggled, this, &DisplacementLimitSettingWidget::onShowModelRadioButtonToggled);
    connect(ui->uxShowRadio, &QRadioButton::toggled, this, &DisplacementLimitSettingWidget::onShowModelRadioButtonToggled);
    connect(ui->uyShowRadio, &QRadioButton::toggled, this, &DisplacementLimitSettingWidget::onShowModelRadioButtonToggled);
    connect(ui->uzShowRadio, &QRadioButton::toggled, this, &DisplacementLimitSettingWidget::onShowModelRadioButtonToggled);

    // 点选开关
    connect(ui->clickSelectionCheckBox, &QCheckBox::toggled, this, &DisplacementLimitSettingWidget::onSelectModelToggled);

    //点半径的slider
    connect(ui->radiusSlider, &QSlider::valueChanged, this, &DisplacementLimitSettingWidget::radiusChange);
}

DisplacementLimitSettingWidget::~DisplacementLimitSettingWidget()
{
    delete ui;
}

void DisplacementLimitSettingWidget::showData(int id,bool enable[3],double value[3])
{
    ui->IDLabel->setText(QString::number(id+1));
    if(enable[0])
    {
        ui->UXLabel->setText(QString::number(value[0]));
    }
    else
    {
        ui->UXLabel->setText(QStringLiteral("未设置"));
    }
    if(enable[1])
    {
        ui->UYLabel->setText(QString::number(value[1]));
    }
    else
    {
        ui->UYLabel->setText(QStringLiteral("未设置"));
    }
    if(enable[2])
    {
        ui->UZLabel->setText(QString::number(value[2]));
    }
    else
    {
        ui->UZLabel->setText(QStringLiteral("未设置"));
    }
}

void DisplacementLimitSettingWidget::radiusChange(int value)
{
    value -= 5;
    if(value == 0)
    {
        return;
    }
    else if(value > 0)
    {
        m_sphereSource->SetRadius(m_radius*value);
    }
    else
    {
        m_sphereSource->SetRadius(m_radius/abs(value));
    }
    m_visualWidgetService->getrenderWindow()->Render();
}

void DisplacementLimitSettingWidget::showEvent(QShowEvent *event)
{
    if(m_preProcessingService->isHaveStructured() == false)
    {
        return;
    }
    qDebug()<<QStringLiteral("开始设置位移约束");


    m_boundarySetting->setStructure(m_preProcessingService->getGridStructure(),m_preProcessingService->getPointStructure());
    m_boundarySetting->setFaceToPoint(m_preProcessingService->getFaceToPoint());
    
    m_boundarySetting->set3DType(m_preProcessingService->is3DType());
    
    m_visualWidgetService->addActor(m_preProcessingService->getPointActor());

    m_sphereSource = m_preProcessingService->getSphereSource();
    m_radius = m_sphereSource->GetRadius();
    ui->radiusSlider->setRange(0, 10);
    ui->radiusSlider->setValue(5);


    m_visualWidgetService->getrenderWindow()->Render();

    m_boundarySetting->setBoundarySettingStatus(1);

    ui->comboBox->clear();
    for(int i=1;i<m_preProcessingService->getFaceToPoint().size();i++)
    {
        ui->comboBox->addItem(QString::number(i));
    }

    ui->allShowRadio->setChecked(true);
    ui->clickSelectionCheckBox->setChecked(true);
}

void DisplacementLimitSettingWidget::hideEvent(QHideEvent *event)
{
    if(m_preProcessingService->isHaveStructured() == false)
    {
        return;
    }

    qDebug()<<QStringLiteral("结束设置位移约束");
    //--------------清除
    m_boundarySetting->clearSelectedPoint();
    m_boundarySetting->setBoundarySettingStatus(0);
    m_visualWidgetService->removeActor(m_preProcessingService->getPointActor());
    m_visualWidgetService->getrenderWindow()->Render();

}

//设置显示方式
void DisplacementLimitSettingWidget::onShowModelRadioButtonToggled(bool checked)
{
    if(!checked) return;
    // Sender() 获取触发信号的对象
    QRadioButton *button = qobject_cast<QRadioButton*>(sender());

    if (button) {
        if(button->text() == QStringLiteral("全部"))
        {
            m_boundarySetting->setPointShowModel(0);
        }
        else if(button->text() == QStringLiteral("UX"))
        {
            m_boundarySetting->setPointShowModel(1);
        }
        else if(button->text() == QStringLiteral("UY"))
        {
            m_boundarySetting->setPointShowModel(2);
        }
        else if(button->text() == QStringLiteral("UZ"))
        {
            m_boundarySetting->setPointShowModel(3);
        }
    }
}

void DisplacementLimitSettingWidget::onSelectModelToggled(bool checked)
{
    if(checked)
    {
        m_boundarySetting->setBoundarySettingStatus(1);
    }
    else
    {
        m_boundarySetting->setBoundarySettingStatus(0);
    }
}

//设置值
void DisplacementLimitSettingWidget::on_setPointData_clicked()
{
    // 使用正则表达式检查是否仅包含有效数字
    QRegularExpression re("^\\d*(\\.\\d+)?$");


    bool enable[3]={false,false,false};
    double value[3];
    if(ui->UXCheckBox->isChecked())
    {
        enable[0] = true;

        QString text = ui->UXLineEdit->text().trimmed(); // 去掉前后空格
        QRegularExpressionMatch match = re.match(text);

        if (match.hasMatch()) {
            if(text == "")
            {
                QMessageBox::warning(nullptr, QStringLiteral("输入错误"), QStringLiteral("请输入有效的数字。"));
                return;
            }
            value[0] = text.toDouble();
        } else {
            QMessageBox::warning(nullptr, QStringLiteral("输入错误"), QStringLiteral("请输入有效的数字。"));
            return;
        }
    }
    if(ui->UYCheckBox->isChecked())
    {
        enable[1] = true;

        QString text = ui->UYLineEdit->text().trimmed(); // 去掉前后空格
        QRegularExpressionMatch match = re.match(text);

        if (match.hasMatch()) {
            if(text == "")
            {
                QMessageBox::warning(nullptr, QStringLiteral("输入错误"), QStringLiteral("请输入有效的数字。"));
                return;
            }
            value[1] = text.toDouble();
        } else {
            QMessageBox::warning(nullptr, QStringLiteral("输入错误"), QStringLiteral("请输入有效的数字。"));
            return;
        }
    }
    if(ui->UZCheckBox->isChecked())
    {
        enable[2] = true;
        QString text = ui->UZLineEdit->text().trimmed(); // 去掉前后空格


        QRegularExpressionMatch match = re.match(text);

        if (match.hasMatch()) {
            if(text == "")
            {
                QMessageBox::warning(nullptr, QStringLiteral("输入错误"), QStringLiteral("请输入有效的数字。"));
                return;
            }
            value[2] = text.toDouble();
        } else {
            QMessageBox::warning(nullptr, QStringLiteral("输入错误"), QStringLiteral("请输入有效的数字。"));
            return;
        }
    }
    if(!enable[0]&&!enable[1]&&!enable[2])
    {
        QMessageBox::warning(nullptr, QStringLiteral("输入错误"), QStringLiteral("请选择有效的参数。"));
        return;
    }
    m_boundarySetting->setPointData(enable,value);
}


void DisplacementLimitSettingWidget::on_deletePointData_clicked()
{
    m_boundarySetting->deletePointData();
}

//清除选中的点
void DisplacementLimitSettingWidget::on_clearSelectedButton_clicked()
{
    m_boundarySetting->clearSelectedPoint();
}


void DisplacementLimitSettingWidget::on_batchSelectButton_clicked()
{
    m_boundarySetting->batchSelectPoint(ui->comboBox->currentIndex()+1);
}

