#include "PhysicalField.h"
#include <utils/ServiceRegistry.h>
#include <QToolButton>
#include <QUuid>

PhysicalField::PhysicalField()
{
    m_boundarySetting = new BoundarySetting();
    m_projectManagerService = ServiceRegistry<ProjectManagerService>::instance().getService();
    m_uiFrameService = ServiceRegistry<UiFrameService>::instance().getService();
    ServiceRegistry<PhysicalFieldService>::instance().registerService(this);
    setupTreeWidget();
    m_boundarySetting->setPhyicalField(this);

}

void PhysicalField::setupTreeWidget()
{
    //添加tab按钮
    QToolButton *analysisTypeButton = new QToolButton;
    analysisTypeButton->setText(QStringLiteral("分析类型"));
    analysisTypeButton->setIcon(QIcon(":/resources/menu/analysisType.png"));
     m_uiFrameService->addMenuButton(QStringLiteral("物理场"), "help", analysisTypeButton);

    QToolButton *boundrySettingButton = new QToolButton;
    boundrySettingButton->setText(QStringLiteral("边界设置"));
    boundrySettingButton->setIcon(QIcon(":/resources/menu/boundarySettings.png"));
     m_uiFrameService->addMenuButton(QStringLiteral("物理场"), "help", boundrySettingButton);

    QToolButton *generalStaticButton = new QToolButton;
    generalStaticButton->setText(QStringLiteral("一般静力"));
    generalStaticButton->setIcon(QIcon(":/resources/menu/generalStatic.png"));
     m_uiFrameService->addMenuButton(QStringLiteral("物理场"), "help", generalStaticButton);

    QToolButton *excavationButton = new QToolButton;
    excavationButton->setText(QStringLiteral("开挖问题"));
    excavationButton->setIcon(QIcon(":/resources/menu/Excavation.png"));
    m_uiFrameService->addMenuButton(QStringLiteral("物理场"), "help", excavationButton);



    //添加settingWidget
    m_analysisTypeSettingWidget = new AnalysisTypeSettingWidget(this);
//    m_boundarySettingWidget = new BoundarySettingWidget(this);
    m_displacementLimitSettingWidget = new DisplacementLimitSettingWidget(this);
    m_distributedLoadSettingWidget = new DistributedLoadSettingWidget(this);

    m_staticProblemsSettingWidget = new StaticProblemsSettingWidget(this);
    m_excavateSettingWidget = new ExcavateSettingWidget(this);

    QString uuid =  QUuid::createUuid().toString();
    QTreeWidgetItem *rootItem = new QTreeWidgetItem();
    rootItem->setText(0, QStringLiteral("物理场"));
    rootItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->addTreeItem(rootItem);
    m_projectManagerService->setItemWidget(uuid,m_analysisTypeSettingWidget);

    connect(analysisTypeButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(rootItem);
    });


    QTreeWidgetItem *childItem = nullptr;

//    childItem = new QTreeWidgetItem(rootItem);
//    childItem->setText(0, QStringLiteral("边界设置"));
//    uuid =  QUuid::createUuid().toString();
//    childItem->setData(0, Qt::UserRole, uuid);
//    m_projectManagerService->setItemWidget(uuid,m_boundarySettingWidget);

//    connect(boundrySettingButton, &QToolButton::clicked, [=]() {
//            m_projectManagerService->setCurrentItem(childItem);
//    });

    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("位移约束"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_displacementLimitSettingWidget);

//    connect(boundrySettingButton, &QToolButton::clicked, [=]() {
//            m_projectManagerService->setCurrentItem(childItem);
//    });

    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("均布载荷"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_distributedLoadSettingWidget);

//    connect(boundrySettingButton, &QToolButton::clicked, [=]() {
//            m_projectManagerService->setCurrentItem(childItem);
//    });


    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("一般静力问题"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_staticProblemsSettingWidget);

    connect(generalStaticButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(childItem);
    });


    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("开挖"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_excavateSettingWidget);

    connect(excavationButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(childItem);
    });
}

BoundarySetting *PhysicalField::getBoundarySetting()
{
    return m_boundarySetting;
}

DisplacementLimitSettingWidget *PhysicalField::getDisplacementLimitSettingWidget()
{
    return m_displacementLimitSettingWidget;
}

DistributedLoadSettingWidget *PhysicalField::getDistributedLoadSettingWidget()
{
    return m_distributedLoadSettingWidget;
}
