#include "PropPickerInteractionStyle.h"
#include "utils/ServiceRegistry.h"
#include "BoundarySetting.h"
#include "vtkRenderWindowInteractor.h"
#include <QDebug>
#include <vtkCellPicker.h>
#include<vtkPoints.h>
#include<vtkPolyData.h>
#include<vtkCellData.h>
#include<vtkCellArray.h>
#include<vtkQuad.h>
#include<vtkRenderWindowInteractor.h>
#include<vtkHexahedron.h>
#include <vtkAutoInit.h>
#include<vtkUnstructuredGrid.h>
#include<vtkDataSetMapper.h>
#include <QToolButton>
#include <vtkCellPicker.h>
#include <vtkPointPicker.h>
#include <vtkVertexGlyphFilter.h>
#include <vtkSphereSource.h>
#include <set>
#include <vtkCell.h>
#include <vtkIdList.h>
#include <vtkDoubleArray.h>
#include <vtkMapper.h>
#include <vtkInformation.h>
#include <vtkInformationObjectBaseVectorKey.h>
#include <QFile>
#include <vtkQuad.h>
#include <vtkMath.h>
#include <vtkLine.h>
#include <vtkCellArray.h>
#include <vtkLODActor.h>
#include <vtkAppendPolyData.h>
#include <vtkGlyph3D.h>
#include <vtkPointData.h>
#include <vtkCleanPolyData.h>
#include <vtkTextActor.h>
#include <vtkTextProperty.h>
#include <vtkVectorText.h>
#include <vtkLightCollection.h>
#include <vtkCollectionIterator.h>
#include <vtkLight.h>
#include <vtkFollower.h>
#include <public/struct/Mesh.h>
#include <vtkRendererCollection.h>
#include <public/services/PreProcessingService.h>

PropPickerInteractionStyle::PropPickerInteractionStyle(BoundarySetting *boundarySetting)
{
    m_boundarySetting = boundarySetting;
    m_visualWidgetService = ServiceRegistry<VisualWidgetService>::instance().getService();
    m_is3DType = true;
}

void PropPickerInteractionStyle::Execute(vtkObject *caller, unsigned long eventId, void *callData)
{
    if (eventId == vtkCommand::LeftButtonPressEvent)
    {
        if(m_selectModel == 0)
        {
            return;
        }
        if(m_selectModel == 1)
        {
            selectGridNode(caller, eventId, callData);
        }
        else
        {
            selectGridFace(caller, eventId, callData);
        }
    }
}


void PropPickerInteractionStyle::setSelectModel(int selectModel)
{
    m_selectModel = selectModel;
    if(m_selectModel == 0)
    {
        qDebug()<<QStringLiteral("???");
    }
    else if(m_selectModel == 1)
    {
        qDebug()<<QStringLiteral("??");
    }
    else if(m_selectModel == 2)
    {
        qDebug()<<QStringLiteral("??");
    }
}

QSet<int> PropPickerInteractionStyle::getSelectedPoint()
{
    return m_selectedPoint;
}

QSet<int> PropPickerInteractionStyle::getSelectedFace()
{
    return m_selectedFace;
}


void PropPickerInteractionStyle::addSelectedPoint(int pointId)
{
    m_selectedPoint.insert(pointId);
}

void PropPickerInteractionStyle::addSelectedFace(int faceId)
{
    m_selectedFace.insert(faceId);
}

void PropPickerInteractionStyle::clearSelectedPoint()
{
    m_selectedPoint.clear();
}

void PropPickerInteractionStyle::clearSelectedFace()
{
    m_selectedFace.clear();
}

void PropPickerInteractionStyle::set3DType(bool is3DType)
{
    m_is3DType = is3DType;
}

void PropPickerInteractionStyle::setPointColor(QSet<int>point,double color[3])
{
    vtkDataArray *pointColor = m_boundarySetting->getPointStructure()->GetPointData()->GetScalars();
    for (const int &pt : point) {
        pointColor->SetComponent(pt, 0, color[0]);
        pointColor->SetComponent(pt, 1, color[1]);
        pointColor->SetComponent(pt, 2, color[2]);
    }
    pointColor->Modified();
    m_visualWidgetService->getrenderWindow()->Render();
}

void PropPickerInteractionStyle::setFaceColor(QSet<int>face,double color[3])
{
    vtkDataArray *faceColor = m_boundarySetting->getCellStructure()->GetCellData()->GetScalars();
    for (const int &pt : face) {       
        faceColor->SetComponent(pt, 0, color[0]);
        faceColor->SetComponent(pt, 1, color[1]);
        faceColor->SetComponent(pt, 2, color[2]);
    }
    faceColor->Modified();
    m_visualWidgetService->getrenderWindow()->Render();
}

void PropPickerInteractionStyle::setPointColor(int pointId,double color[3])
{
    vtkDataArray *pointColor = m_boundarySetting->getPointStructure()->GetPointData()->GetScalars();
    pointColor->SetComponent(pointId, 0, color[0]);
    pointColor->SetComponent(pointId, 1, color[1]);
    pointColor->SetComponent(pointId, 2, color[2]);
}

void PropPickerInteractionStyle::setFaceColor(int faceId,double color[3])
{
    vtkDataArray *faceColor = m_boundarySetting->getCellStructure()->GetCellData()->GetScalars();
    faceColor->SetComponent(faceId, 0, color[0]);
    faceColor->SetComponent(faceId, 1, color[1]);
    faceColor->SetComponent(faceId, 2, color[2]);
}

void PropPickerInteractionStyle::pointRefresh()
{
    vtkDataArray *pointColor = m_boundarySetting->getPointStructure()->GetPointData()->GetScalars();
    pointColor->Modified();
    m_visualWidgetService->getrenderWindow()->Render();
}

void PropPickerInteractionStyle::faceRefresh()
{
    vtkDataArray *faceColor = m_boundarySetting->getCellStructure()->GetCellData()->GetScalars();
    faceColor->Modified();
    m_visualWidgetService->getrenderWindow()->Render();
}


void PropPickerInteractionStyle::clearPointColor()
{
    vtkDataArray *pointColor = m_boundarySetting->getPointStructure()->GetPointData()->GetScalars();
    pointColor->FillComponent(0, 255);
    pointColor->FillComponent(1, 255);
    pointColor->FillComponent(2, 255);
    pointColor->Modified();
    m_visualWidgetService->getrenderWindow()->Render();
}

void PropPickerInteractionStyle::clearFaceColor()
{
    vtkDataArray *faceColor = m_boundarySetting->getCellStructure()->GetCellData()->GetScalars();
    if(m_is3DType)
    {
        faceColor->FillComponent(0, 255);
        faceColor->FillComponent(1, 255);
        faceColor->FillComponent(2, 255);
    }
    else
    {
        faceColor->FillComponent(0, 0);
        faceColor->FillComponent(1, 0);
        faceColor->FillComponent(2, 0);
    }
    faceColor->Modified();
    m_visualWidgetService->getrenderWindow()->Render();
}

void PropPickerInteractionStyle::selectGridNode(vtkObject *caller, unsigned long eventId, void *callData)
{
    if(m_is3DType==0){
    vtkRenderWindowInteractor* interactor = vtkRenderWindowInteractor::SafeDownCast(caller);
    if (!interactor)
        return;

    // ??????????
    int* clickPos = interactor->GetEventPosition();

    // ??
    vtkSmartPointer<vtkCellPicker> picker = vtkSmartPointer<vtkCellPicker>::New();
    picker->SetTolerance(0.001);
    picker->Pick(clickPos[0], clickPos[1], 0, interactor->GetRenderWindow()->GetRenderers()->GetFirstRenderer());

    vtkUnstructuredGrid *polyData = m_boundarySetting->getPointStructure();
    int cellId = picker->GetCellId();
    qDebug()<<"cellid "<<cellId;
    if (cellId != -1) {

        vtkCell* cell = polyData->GetCell(cellId);
        vtkPoints* cellPoints = cell->GetPoints();

        double clickPos[3];
        picker->GetPickPosition(clickPos);

        //??????????
        double minDist = VTK_DOUBLE_MAX;
        vtkIdType closestPointIdInCell = -1;
        vtkIdType closestPointIdInPolyData = -1;

        for (vtkIdType i = 0; i < cellPoints->GetNumberOfPoints(); ++i) {
            double point[3];
            cellPoints->GetPoint(i, point);

            double dist = vtkMath::Distance2BetweenPoints(clickPos, point);
            if (dist < minDist) {
                minDist = dist;
                closestPointIdInCell = i;
                closestPointIdInPolyData = cell->GetPointId(i);
            }
        }

        if (closestPointIdInPolyData != -1) {
            double closestPoint[3];
            polyData->GetPoints()->GetPoint(closestPointIdInPolyData, closestPoint);
            qDebug() <<"point: "<<closestPointIdInPolyData+1;
            qDebug() << "Closest point coordinates: ("
                     << closestPoint[0] << ", "
                     << closestPoint[1] << ", "
                     << closestPoint[2] << ")";

            vtkDataArray *pointColor = polyData->GetPointData()->GetScalars();

            //???????????????
            if(m_selectedPoint.contains(closestPointIdInPolyData))
            {
                m_selectedPoint.remove(closestPointIdInPolyData);
                if(m_boundarySetting->isSelectedPoint(closestPointIdInPolyData))//?????
                {
                    pointColor->SetComponent(closestPointIdInPolyData, 0, 255);
                    pointColor->SetComponent(closestPointIdInPolyData, 1, 0);
                    pointColor->SetComponent(closestPointIdInPolyData, 2, 255);
                    pointColor->Modified();
                }
                else //??????
                {
                    pointColor->SetComponent(closestPointIdInPolyData, 0, 255);
                    pointColor->SetComponent(closestPointIdInPolyData, 1, 255);
                    pointColor->SetComponent(closestPointIdInPolyData, 2, 255);
                    pointColor->Modified();
                }
            }
            else
            {
                m_selectedPoint.insert(closestPointIdInPolyData);
                pointColor->SetComponent(closestPointIdInPolyData, 0, 255);
                pointColor->SetComponent(closestPointIdInPolyData, 1, 0);
                pointColor->SetComponent(closestPointIdInPolyData, 2, 0);
                pointColor->Modified();
            }

            m_boundarySetting->showPointData(closestPointIdInPolyData);
        }
    }}
    else{
    vtkRenderWindowInteractor* interactor = vtkRenderWindowInteractor::SafeDownCast(caller);
    if (!interactor)
        return;

    // ??????????
    int* clickPos = interactor->GetEventPosition();

    // ??
    vtkSmartPointer<vtkPointPicker> picker = vtkSmartPointer<vtkPointPicker>::New();
    picker->SetTolerance(0.001);
    picker->Pick(clickPos[0], clickPos[1], 0, interactor->GetRenderWindow()->GetRenderers()->GetFirstRenderer());

    vtkUnstructuredGrid *polyData = m_boundarySetting->getPointStructure();
    vtkIdType closestPointIdInPolyData = picker->GetPointId();
    std::cout<<"111111111111111111111111111111"<<std::endl;
    qDebug()<<"cellid "<<(int)(closestPointIdInPolyData);
    //qDebug()<<"cellid "<<cellId;
    //if (cellId != -1) {

        if (closestPointIdInPolyData != -1) {
            double closestPoint[3];
            polyData->GetPoints()->GetPoint(closestPointIdInPolyData, closestPoint);
            qDebug() <<"point: "<<closestPointIdInPolyData+1;
            qDebug() << "Closest point coordinates: ("
                     << closestPoint[0] << ", "
                     << closestPoint[1] << ", "
                     << closestPoint[2] << ")";

            vtkDataArray *pointColor = polyData->GetPointData()->GetScalars();

            //???????????????
            if(m_selectedPoint.contains(closestPointIdInPolyData))
            {
                m_selectedPoint.remove(closestPointIdInPolyData);
                if(m_boundarySetting->isSelectedPoint(closestPointIdInPolyData))//?????
                {
                    pointColor->SetComponent(closestPointIdInPolyData, 0, 255);
                    pointColor->SetComponent(closestPointIdInPolyData, 1, 0);
                    pointColor->SetComponent(closestPointIdInPolyData, 2, 255);
                    pointColor->Modified();
                }
                else //??????
                {
                    pointColor->SetComponent(closestPointIdInPolyData, 0, 255);
                    pointColor->SetComponent(closestPointIdInPolyData, 1, 255);
                    pointColor->SetComponent(closestPointIdInPolyData, 2, 255);
                    pointColor->Modified();
                }
            }
            else
            {
                m_selectedPoint.insert(closestPointIdInPolyData);
                pointColor->SetComponent(closestPointIdInPolyData, 0, 255);
                pointColor->SetComponent(closestPointIdInPolyData, 1, 0);
                pointColor->SetComponent(closestPointIdInPolyData, 2, 0);
                pointColor->Modified();
            }

            m_boundarySetting->showPointData(closestPointIdInPolyData);
        //}
    }
    }
}

void PropPickerInteractionStyle::selectGridFace(vtkObject *caller, unsigned long eventId, void *callData)
{
    vtkUnstructuredGrid *polyData = m_boundarySetting->getCellStructure();
    vtkRenderWindowInteractor* interactor = vtkRenderWindowInteractor::SafeDownCast(caller);
    vtkSmartPointer<vtkCellPicker> picker = vtkSmartPointer<vtkCellPicker>::New();
    picker->SetTolerance(0.001);
    int* clickPos = interactor->GetEventPosition();
    picker->Pick(clickPos[0], clickPos[1], 0, interactor->GetRenderWindow()->GetRenderers()->GetFirstRenderer());

    int cellId = picker->GetCellId();

    if (cellId != -1) {
        // Access the cell using the cellId
        vtkCell* cell = polyData->GetCell(cellId);

        // ??? cell ??? ID ???
        vtkIdList* pointIds = cell->GetPointIds();
        std::cout << "Cell ID: " << cellId+1 << std::endl;
        //std::cout << "Grid ID:" << m_boundarySetting->getFaceIdToGridId(cellId) <<std::endl;

        // ?????????
        vtkPoints* points = polyData->GetPoints();
        for (vtkIdType i = 0; i < pointIds->GetNumberOfIds(); ++i) {
            vtkIdType pointId = pointIds->GetId(i);
            double point[3];
            points->GetPoint(pointId, point);

            std::cout << "Point ID: " <<  pointId+1 << " Coordinates: ("
                      << point[0] << ", " << point[1] << ", " << point[2] << ")"
                      << std::endl;
        }
        double col[3]{255, 0, 0};
        vtkDataArray *cellcolor = polyData->GetCellData()->GetScalars();


        //???????????????
        if(m_selectedFace.contains(cellId))
        {
            m_selectedFace.remove(cellId);
            if(m_boundarySetting->isSelectedFace(cellId))//?????
            {
                cellcolor->SetComponent(cellId, 0, 255);
                cellcolor->SetComponent(cellId, 1, 0);
                cellcolor->SetComponent(cellId, 2, 255);
                cellcolor->Modified();
            }
            else //??????
            {
                if(m_is3DType)
                {
                    cellcolor->SetComponent(cellId, 0, 255);
                    cellcolor->SetComponent(cellId, 1, 255);
                    cellcolor->SetComponent(cellId, 2, 255);
                }
                else
                {
                    cellcolor->SetComponent(cellId, 0, 0);
                    cellcolor->SetComponent(cellId, 1, 0);
                    cellcolor->SetComponent(cellId, 2, 0);
                }
               
                cellcolor->Modified();
            }
        }
        else
        {
            m_selectedFace.insert(cellId);
            cellcolor->SetComponent(cellId, 0, 255);
            cellcolor->SetComponent(cellId, 1, 0);
            cellcolor->SetComponent(cellId, 2, 0);
            cellcolor->Modified();
        }

        m_boundarySetting->showFaceData(cellId);
    }
}
