#include "NephogramSettingWidget.h"
#include "PostProcessing.h"
#include "ui_NephogramSettingWidget.h"

#include <utils/ServiceRegistry.h>

#include <public/services/VisualWidgetService.h>

#include <QDir>

NephogramSettingWidget::NephogramSettingWidget(PostProcessing *postProcessing, QWidget *parent) :
    QWidget(parent),
    m_postProcessing(postProcessing),
    ui(new Ui::NephogramSettingWidget)
{
    ui->setupUi(this);
    m_visualWidgetService = ServiceRegistry<VisualWidgetService>::instance().getService();
    connect(ui->comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal2(int)));

    m_isFirstClick = true;
}

void NephogramSettingWidget::CheckIndexChangedSignal2(int i)
{
    disconnect(ui->comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal(int)));
    ui->comboBox->clear();

    m_visualWidgetService->removeAllActor();
    readFile();
    connect(ui->comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal(int)));
}

NephogramSettingWidget::~NephogramSettingWidget()
{
    delete ui;
}

void NephogramSettingWidget::CheckIndexChangedSignal(int i)
{
    if(m_type == 0)
    {
        m_postProcessing->nodeChange(i);
    }
    else
        m_postProcessing->unitChange(i);

    m_visualWidgetService->reFreash();
}

void NephogramSettingWidget::setNephogramType(std::vector<std::string> vec_ps)
{
    ui->comboBox->clear();
    for (int i=0;i<vec_ps.size();i++)
    {
        ui->comboBox->addItem(QString::fromStdString(vec_ps[i]));
    }
    connect(ui->comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal(int)));
}

//开始读


void NephogramSettingWidget::readFile()
{
    m_visualWidgetService->removeAllActor();
    ui->comboBox->clear();
    int currentIndex = ui->comboBox_2->currentIndex();
    QString selectedText = ui->comboBox_2->itemText(currentIndex);
    std::vector<std::string> vec_ps;

    if(selectedText == "") return;

    if(m_type == 0)
    {
        QString str = "./solver/data/Elem_contour0-"+selectedText+".econ";
        qDebug()<<str;
        m_postProcessing->readNodeNephogram(vec_ps,str.toStdString());
        // 遍历 vec_ps，将每个字符串插入到 QComboBox 中
            for (const std::string &str : vec_ps) {
                QString qstr = QString::fromStdString(str);
                ui->comboBox->addItem(qstr);
            }
    }
    else
    {
        QString str = "./solver/data/stress_strain0-"+selectedText+".txt";
        m_postProcessing->readUnitNephogram(vec_ps,str.toStdString());
        // 遍历 vec_ps，将每个字符串插入到 QComboBox 中
            for (const std::string &str : vec_ps) {
                QString qstr = QString::fromStdString(str);
                ui->comboBox->addItem(qstr);
            }
    }
    connect(ui->comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal(int)));
    CheckIndexChangedSignal(0);
}

//unit
void NephogramSettingWidget::on_radioButton_clicked()
{
    disconnect(ui->comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal(int)));
    disconnect(ui->comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal2(int)));


    m_type = 1;
    ui->comboBox_2->clear();
    ui->comboBox->clear();
    m_visualWidgetService->removeAllActor();
    QString folderPath = "./solver/data"; // 替换为你的文件夹路径
        QVector<int> numbers;

        QDir directory(folderPath);

        if (!directory.exists()) {
            qDebug() << "Directory not found.";
        }

        QString baseName = "stress_strain0-";
        QString extension = ".txt";

        // 构建文件名的正则表达式，用于匹配序号部分
        QString regexStr = baseName + "(\\d+)" + extension;
        QRegularExpression regex(regexStr);

        // 获取文件夹中的所有文件
        QStringList files = directory.entryList(QDir::Files);

        // 遍历文件列表，提取序号
        foreach (QString filename, files) {
            QString filePath = directory.filePath(filename);

            // 匹配正则表达式
            QRegularExpressionMatch match = regex.match(filename);

            if (match.hasMatch()) {
                QString numberStr = match.captured(1); // 获取匹配的第一个捕获组，即序号部分

                // 将提取的序号转换为整数并存入 QVector
                bool ok;
                int number = numberStr.toInt(&ok);
                if (ok) {
                    numbers.append(number);
                }
            }
        }

        std::sort(numbers.begin(), numbers.end());
        // 打印 QVector 中的所有序号，作为验证
        qDebug() << "Extracted numbers from filenames:";
        for (int num : numbers) {
            ui->comboBox_2->addItem(QString::number(num));
        }
        readFile();
        connect(ui->comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal2(int)));
}

//node
void NephogramSettingWidget::on_radioButton_2_clicked()
{
    disconnect(ui->comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal(int)));
    disconnect(ui->comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal2(int)));
        m_type = 0;

        ui->comboBox_2->clear();
        ui->comboBox->clear();
        m_visualWidgetService->removeAllActor();
        QString folderPath = "./solver/data"; // 替换为你的文件夹路径
            QVector<int> numbers;

            QDir directory(folderPath);

            if (!directory.exists()) {
                qDebug() << "Directory not found.";
            }

            QString baseName = "Elem_contour0-";
            QString extension = ".econ";

            // 构建文件名的正则表达式，用于匹配序号部分
            QString regexStr = baseName + "(\\d+)" + extension;
            QRegularExpression regex(regexStr);

            // 获取文件夹中的所有文件
            QStringList files = directory.entryList(QDir::Files);

            // 遍历文件列表，提取序号
            foreach (QString filename, files) {
                QString filePath = directory.filePath(filename);

                // 匹配正则表达式
                QRegularExpressionMatch match = regex.match(filename);

                if (match.hasMatch()) {
                    QString numberStr = match.captured(1); // 获取匹配的第一个捕获组，即序号部分

                    // 将提取的序号转换为整数并存入 QVector
                    bool ok;
                    int number = numberStr.toInt(&ok);
                    if (ok) {
                        numbers.append(number);
                    }
                }
            }

            std::sort(numbers.begin(), numbers.end());
            // 打印 QVector 中的所有序号，作为验证
            qDebug() << "Extracted numbers from filenames:";
            for (int num : numbers) {
                ui->comboBox_2->addItem(QString::number(num));
            }
            readFile();
            connect(ui->comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(CheckIndexChangedSignal2(int)));
}

