#include "PostProcessing.h"

#include <QToolButton>
#include <QTreeWidgetItem>
#include <QUuid>
#include <vtkTextProperty.h>
#include <utils/ServiceRegistry.h>

PostProcessing::PostProcessing()
{
    m_projectManagerService = ServiceRegistry<ProjectManagerService>::instance().getService();
    m_visualWidgetService = ServiceRegistry<VisualWidgetService>::instance().getService();
    m_uiFrameService = ServiceRegistry<UiFrameService>::instance().getService();
    setupTreeWidget();
    m_vtkScalar = new VTKScalar();
}

void PostProcessing::readUnitNephogram(std::vector<std::string> &vec_ps,std::string inputFilename)
{
    std::vector<std::string> vec_ps2;
    m_nephoGramActor = m_vtkScalar->readScalarFile(inputFilename,vec_ps,m_grid,m_scalarBar);
    const vtkNew<vtkTextProperty> labelProperty;
    labelProperty->BoldOn();
    labelProperty->SetColor(0.5, 0.5, 0.5);
    labelProperty->SetOpacity(1);
    labelProperty->SetFontSize(13);

    m_scalarBar->SetLabelTextProperty(labelProperty);
    //一定要设置这个，这个是核心，不设置这个会无效
    m_scalarBar->SetUnconstrainedFontSize(true);
    // 设置数字格式为科学计数法，保留2位有效数字
    m_scalarBar->SetLabelFormat("%-#6.0e");
    m_scalarBar->SetWidth( 0.06 );
    m_scalarBar->SetHeight( 0.7 );
    m_scalarBar->SetPosition(0.9, 0.2);
    vtkTextProperty* textProp = m_scalarBar->GetLabelTextProperty();

    m_visualWidgetService->addActor(m_nephoGramActor);
    m_visualWidgetService->addActor2D(m_scalarBar);
}

void PostProcessing::readNodeNephogram(std::vector<std::string> &vec_ps,std::string inputFilename)
{
    std::vector<std::string> vec_ps2;
    m_nephoGramActor = m_vtkScalar->readScalarFile2(inputFilename,vec_ps,m_grid,m_scalarBar);


    const vtkNew<vtkTextProperty> labelProperty;
    labelProperty->BoldOn();
    labelProperty->SetColor(0.5, 0.5, 0.5);
    labelProperty->SetOpacity(1);
    labelProperty->SetFontSize(13);

    m_scalarBar->SetLabelTextProperty(labelProperty);
    //一定要设置这个，这个是核心，不设置这个会无效
    m_scalarBar->SetUnconstrainedFontSize(true);
    // 设置数字格式为科学计数法，保留2位有效数字
    m_scalarBar->SetLabelFormat("%-#6.0e");
    m_scalarBar->SetWidth( 0.06 );
    m_scalarBar->SetHeight( 0.7 );
    m_scalarBar->SetPosition(0.9, 0.2);
    vtkTextProperty* textProp = m_scalarBar->GetLabelTextProperty();

    m_visualWidgetService->addActor(m_nephoGramActor);
    m_visualWidgetService->addActor2D(m_scalarBar);
}

void PostProcessing::unitChange(int i)
{
    m_vtkScalar->change(i);
}

void PostProcessing::nodeChange(int i)
{
    m_vtkScalar->change2(i);
}

void PostProcessing::setupTreeWidget()
{
    //--------------结果

    QToolButton *nephogramButton = new QToolButton;
    nephogramButton->setText(QStringLiteral("云图"));
    nephogramButton->setIcon(QIcon(":/resources/menu/nephogram.svg"));
    m_uiFrameService->addMenuButton(QStringLiteral("结果"), "help", nephogramButton);

    QToolButton *curveButton = new QToolButton;
    curveButton->setText(QStringLiteral("曲线"));
    curveButton->setIcon(QIcon(":/resources/menu/curve.svg"));
    m_uiFrameService->addMenuButton(QStringLiteral("结果"), "help", curveButton);

    QToolButton *sectionButton = new QToolButton;
    sectionButton->setText(QStringLiteral("切面"));
    sectionButton->setIcon(QIcon(":/resources/menu/section.png"));
    m_uiFrameService->addMenuButton(QStringLiteral("结果"), "help", sectionButton);

    QToolButton *bisectButton = new QToolButton;
    bisectButton->setText(QStringLiteral("剖面"));
    bisectButton->setIcon(QIcon(":/resources/menu/bisect.svg"));
    m_uiFrameService->addMenuButton(QStringLiteral("结果"), "help", bisectButton);


    m_nephogramSettingWidget = new NephogramSettingWidget(this);
    m_graphSettingWidget = new GraphSettingWidget(this);
    m_sectionSettingWidget = new SectionSettingWidget(this);
    m_bisectSettingWidget = new BisectSettingWidget(this);
    m_exportSettingWidget = new ExportSettingWidget(this);

    QString uuid =  QUuid::createUuid().toString();
    QTreeWidgetItem *rootItem = new QTreeWidgetItem();
    rootItem->setText(0, QStringLiteral("后处理"));
    rootItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->addTreeItem(rootItem);
    m_projectManagerService->setItemWidget(uuid,m_exportSettingWidget);


    QTreeWidgetItem *childItem = nullptr;

    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("云图"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_nephogramSettingWidget);
    connect(nephogramButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(childItem);
    });



    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("曲线"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_graphSettingWidget);
    connect(curveButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(childItem);
    });


    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("切面"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_sectionSettingWidget);
    connect(sectionButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(childItem);
    });


    childItem = new QTreeWidgetItem(rootItem);
    childItem->setText(0, QStringLiteral("剖面"));
    uuid =  QUuid::createUuid().toString();
    childItem->setData(0, Qt::UserRole, uuid);
    m_projectManagerService->setItemWidget(uuid,m_bisectSettingWidget);
    connect(bisectButton, &QToolButton::clicked, [=]() {
            m_projectManagerService->setCurrentItem(childItem);
    });

}
