#include<iostream>

#include<vtkProperty.h>
#include<vtkDataSetMapper.h>
#include<vtkUnstructuredGrid.h>
#include<vtkMultiBlockDataSet.h>
#include<vtkPointData.h>
#include<vtkCellData.h>
#include<vtkDataArray.h>
#include<vtkFloatArray.h>
#include<vtkPlane.h>
#include<vtkThreshold.h>

#include "postFilterClip.h"

using namespace std;

postFilterClip::postFilterClip()
{
    m_clip_actor = vtkSmartPointer<vtkActor>::New();
    m_mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    m_lut = vtkSmartPointer<vtkLookupTable>::New();
    m_scalarBar = vtkSmartPointer<vtkScalarBarActor>::New();
    m_ungrid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    m_cell_type = 0;
    m_x_range[2] = {};
    m_y_range[2] = {};
    m_z_range[2] = {};
    m_bound_value[6] = {};
    
    this->buildLut();
    this->buildScalarBar();
}

postFilterClip::~postFilterClip()
{

}
void postFilterClip::setUnstructuredGrid(vtkSmartPointer<vtkUnstructuredGrid> ungrid)
{
    m_ungrid->DeepCopy(ungrid);
    return;
}

void postFilterClip::showProcess(int current_id)
{
    if(m_cell_type==0)
    {
        int m_show_array_id = current_id;
        vtkSmartPointer<vtkTableBasedClipDataSet> clip = vtkSmartPointer<vtkTableBasedClipDataSet>::New();
        m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
        vtkDataArray* array_scalar = m_ungrid->GetPointData()->GetArray(m_show_array_id);
        double* range = array_scalar->GetRange();
        //std::cout<<range[0]<<std::endl;
        //std::cout<<range[1]<<std::endl;
        
        clip->SetInputData(m_ungrid);
        //clip->SetInputData(grid);
        //clip->SetClipFunction(plane);
        //clip->InsideOutOn();
        //clip->GenerateClipScalarsOn();
        //clip->SetValue(15);
        //clip->Update();
   
        //double* range = m_ungrid->GetPointData()->GetArray(m_show_array_id)->GetRange();
    
        m_mapper->SetInputData(m_ungrid);
        m_mapper->SetLookupTable(m_lut);
        m_mapper->SetScalarModeToUsePointFieldData();
        m_mapper->SelectColorArray(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
        m_mapper->SetScalarRange(range); 

        m_clip_actor->SetMapper(m_mapper);
    }
    else if(m_cell_type==1)
    {
        int m_show_array_id = current_id;
        m_ungrid->GetCellData()->SetActiveScalars(m_ungrid->GetCellData()->GetArrayName(m_show_array_id));
        vtkDataArray* array_scalar= m_ungrid->GetCellData()->GetArray(m_show_array_id);
        double* range = array_scalar->GetRange();

        vtkSmartPointer<vtkTableBasedClipDataSet> clip = vtkSmartPointer<vtkTableBasedClipDataSet>::New();
        clip->SetInputData(m_ungrid);
        //clip->SetClipFunction(plane);
        //clip->InsideOutOff();
        //clip->InsideOutOn();
        //clip->GenerateClipScalarsOn();
        //clip->SetValue(-0.01);
        //clip->SetValue(0);
        //clip->Update();

        m_mapper->SetInputData(m_ungrid);
        m_mapper->SetLookupTable(m_lut);
        m_mapper->SetScalarModeToUseCellFieldData();
        m_mapper->SelectColorArray(m_ungrid->GetCellData()->GetArrayName(m_show_array_id));
        m_mapper->SetScalarRange(range); // 强制使用LookupTable的范围
        
        m_clip_actor->SetMapper(m_mapper);

    }
    return;
}

void postFilterClip::update(int current_id,int plane_id,float actua_value,bool b_invert_value)
{
    if(m_cell_type==0)
    {
        updateUsePointData(current_id,plane_id,actua_value,b_invert_value);
    }
    else if(m_cell_type==1)
    {
        updateUseCellData(current_id,plane_id,actua_value,b_invert_value); 
    }
    return;
}

void postFilterClip::updateUsePointData(int current_id,int plane_id,float actual_value,bool b_invert_value)
{
    vtkSmartPointer<vtkPlane> plane = vtkSmartPointer<vtkPlane>::New();
    plane->SetOrigin(0, 0, 0);
    if(plane_id==0)
        plane->SetNormal(1, 0, 0);
    else if(plane_id==1)
        plane->SetNormal(0, 1, 0);
    else if(plane_id==2)
        plane->SetNormal(0, 0, 1);
    
    int m_show_array_id = current_id;
    vtkSmartPointer<vtkTableBasedClipDataSet> clip = vtkSmartPointer<vtkTableBasedClipDataSet>::New();
    m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
    vtkDataArray* array_scalar = m_ungrid->GetPointData()->GetArray(m_show_array_id);
    double* range = array_scalar->GetRange();
    //std::cout<<range[0]<<std::endl;
    //std::cout<<range[1]<<std::endl;
        
    clip->SetInputData(m_ungrid);
    clip->SetClipFunction(plane);
    //clip->InsideOutOn();
    clip->SetInsideOut(b_invert_value);
    //clip->GenerateClipScalarsOn();
    clip->SetValue(actual_value);
    clip->Update();
   
    m_mapper->SetInputData(clip->GetOutput());
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SelectColorArray(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
    m_mapper->SetScalarRange(range); 

    m_clip_actor->SetMapper(m_mapper);

    return;
}

void postFilterClip::updateUseCellData(int current_id,int plane_id,float actua_value,bool b_invert_value)
{
    vtkSmartPointer<vtkPlane> plane = vtkSmartPointer<vtkPlane>::New();
    plane->SetOrigin(0, 0, 0);
    if(plane_id==0)
        plane->SetNormal(1, 0, 0);
    else if(plane_id==1)
        plane->SetNormal(0, 1, 0);
    else if(plane_id==2)
        plane->SetNormal(0, 0, 1);
    
    int m_show_array_id = current_id;
    vtkSmartPointer<vtkTableBasedClipDataSet> clip = vtkSmartPointer<vtkTableBasedClipDataSet>::New();
    m_ungrid->GetCellData()->SetActiveScalars(m_ungrid->GetCellData()->GetArrayName(m_show_array_id));
    vtkDataArray* array_scalar = m_ungrid->GetCellData()->GetArray(m_show_array_id);
    double* range = array_scalar->GetRange();
    //std::cout<<range[0]<<std::endl;
    //std::cout<<range[1]<<std::endl;
        
    clip->SetInputData(m_ungrid);
    clip->SetClipFunction(plane);
    //clip->InsideOutOn();
    clip->SetInsideOut(b_invert_value);
    //clip->GenerateClipScalarsOn();
    clip->SetValue(actua_value);
    clip->Update();
   
    m_mapper->SetInputData(clip->GetOutput());
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUseCellFieldData();
    m_mapper->SelectColorArray(m_ungrid->GetCellData()->GetArrayName(m_show_array_id));
    m_mapper->SetScalarRange(range); 

    m_clip_actor->SetMapper(m_mapper);

    return;
}

float * postFilterClip::getRangeX()
{
    return m_x_range;
}

float * postFilterClip::getRangeY()
{
    return m_y_range;
}

float * postFilterClip::getRangeZ()
{
    return m_z_range;
}

void postFilterClip::setCellType(int cell_type)
{
    m_cell_type = cell_type;
    return;
}

void postFilterClip::initProcess(int cell_type)
{
    setCellType(cell_type);
    m_bound_value[0] = m_ungrid->GetBounds()[0];
    m_bound_value[1] = m_ungrid->GetBounds()[1];
    m_bound_value[2] = m_ungrid->GetBounds()[2];
    m_bound_value[3] = m_ungrid->GetBounds()[3];
    m_bound_value[4] = m_ungrid->GetBounds()[4];
    m_bound_value[5] = m_ungrid->GetBounds()[5];
    
    m_x_range[0] = m_ungrid->GetBounds()[0];
    m_x_range[1] = m_ungrid->GetBounds()[1];
    m_y_range[0] = m_ungrid->GetBounds()[2];
    m_y_range[1] = m_ungrid->GetBounds()[3];
    m_z_range[0] = m_ungrid->GetBounds()[4];
    m_z_range[1] = m_ungrid->GetBounds()[5];

    return;
}

void postFilterClip::buildLut()
{
    // 创建一个色度图
    //vtkSmartPointer<vtkLookupTable> lut = vtkSmartPointer<vtkLookupTable>::New();
    m_lut->SetTableRange(-999, 999);
    m_lut->SetHueRange(0.67, 0.0);
    m_lut->Build();
    return;
}

void postFilterClip::buildScalarBar()
{
    m_scalarBar->SetLookupTable(m_lut);
    m_scalarBar->SetTitle("CSRock");
    m_scalarBar->SetNumberOfLabels(5);
    m_scalarBar->SetPosition(0.9, 0.2);
    m_scalarBar->SetWidth(0.1);
    m_scalarBar->SetHeight(0.6);
    return;
}

vtkSmartPointer<vtkActor> postFilterClip::getActor()
{
    return m_clip_actor;
}

vtkSmartPointer<vtkLookupTable> postFilterClip::getLut()
{
    return m_lut;
}














