#include<iostream>

#include<vtkProperty.h>
#include<vtkDataSetMapper.h>
#include<vtkUnstructuredGrid.h>
#include<vtkMultiBlockDataSet.h>
#include<vtkPointData.h>
#include<vtkDataArray.h>
#include<vtkFloatArray.h>
#include<vtkContourFilter.h>

#include<vtkTecplotReader.h>

#include "postFilterContour.h"

using namespace std;

postFilterContour::postFilterContour()
{
    m_contour_actor = vtkSmartPointer<vtkActor>::New();
    m_mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    m_lut = vtkSmartPointer<vtkLookupTable>::New();
    m_scalarBar = vtkSmartPointer<vtkScalarBarActor>::New();
    m_ungrid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    this->buildLut();
    this->buildScalarBar();
}

postFilterContour::~postFilterContour()
{

}
void postFilterContour::setUnstructuredGrid(vtkSmartPointer<vtkUnstructuredGrid> ungrid)
{
    m_ungrid->DeepCopy(ungrid);
    return;
}

void postFilterContour::showProcess(int current_id)
{
    int m_show_array_id = current_id;
    vtkSmartPointer<vtkContourFilter> contourFilter =
            vtkSmartPointer<vtkContourFilter>::New();
    contourFilter->SetInputData(m_ungrid);
    //contourFilter->GenerateValues(5, range[0],range[1]); 
    //contourFilter->SetValue(0,-0.05); 
    contourFilter->Update();
    
    m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
    vtkDataArray* array_scalar = m_ungrid->GetPointData()->GetArray(m_show_array_id);
    double* range = array_scalar->GetRange();

    //double* range = m_ungrid->GetPointData()->GetArray(m_show_array_id)->GetRange();

    //m_mapper->SetInputData(contourFilter->GetOutput());
    m_mapper->SetInputData(m_ungrid);
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SelectColorArray(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
    m_mapper->SetScalarRange(range); 

    m_contour_actor->SetMapper(m_mapper);
    return;
}

void postFilterContour::initProcess()
{
    return;
}

void postFilterContour::updateMulti(int current_id,float contour_min_value,float contour_max_value,int contour_num)
{
    int show_array_id = current_id;
    //show_array_id = 3;
    m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(show_array_id));
    vtkDataArray* array_scalar = m_ungrid->GetPointData()->GetArray(show_array_id);
    double* range = array_scalar->GetRange();
    //m_lut->SetTableRange(range);

    vtkSmartPointer<vtkContourFilter> contourFilter = vtkSmartPointer<vtkContourFilter>::New();
    contourFilter->SetInputData(m_ungrid);
    contourFilter->GenerateValues(contour_num, contour_min_value,contour_max_value);
    //contourFilter->SetValue(0,contour_value);
    contourFilter->Update();
    
    m_mapper->SetInputData(contourFilter->GetOutput());
    //m_mapper->SetInputConnection(contourFilter->GetOutputPort());
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SelectColorArray(show_array_id);
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range); 
    m_contour_actor->SetMapper(m_mapper);
    m_contour_actor->GetProperty()->SetRepresentationToSurface();
    return;
}

void postFilterContour::updateSingle(int current_id,float contour_value)
{
    int show_array_id = current_id;
    //show_array_id = 3;
    m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(show_array_id));
    vtkDataArray* array_scalar = m_ungrid->GetPointData()->GetArray(show_array_id);
    double* range = array_scalar->GetRange();
    //m_lut->SetTableRange(range);

    vtkSmartPointer<vtkContourFilter> contourFilter = vtkSmartPointer<vtkContourFilter>::New();
    contourFilter->SetInputData(m_ungrid);
    //contourFilter->GenerateValues(5, range[0],range[1]);
    contourFilter->SetValue(0,contour_value);
    contourFilter->Update();
    
    m_mapper->SetInputData(contourFilter->GetOutput());
    //m_mapper->SetInputConnection(contourFilter->GetOutputPort());
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SelectColorArray(show_array_id);
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range); 
    m_contour_actor->SetMapper(m_mapper);
    m_contour_actor->GetProperty()->SetRepresentationToSurface();
    return;
}

void postFilterContour::buildLut()
{
    // 创建一个色度图
    //vtkSmartPointer<vtkLookupTable> lut = vtkSmartPointer<vtkLookupTable>::New();
    m_lut->SetTableRange(-999, 999);
    m_lut->SetHueRange(0.67, 0.0);
    m_lut->Build();
    return;
}

void postFilterContour::buildScalarBar()
{
    m_scalarBar->SetLookupTable(m_lut);
    m_scalarBar->SetTitle("CSRock");
    m_scalarBar->SetNumberOfLabels(5);
    m_scalarBar->SetPosition(0.9, 0.2);
    m_scalarBar->SetWidth(0.1);
    m_scalarBar->SetHeight(0.6);
    return;
}

vtkSmartPointer<vtkActor> postFilterContour::getActor()
{
    return m_contour_actor;
}

vtkSmartPointer<vtkLookupTable> postFilterContour::getLut()
{
    return m_lut;
}














