#include<iostream>

#include<vtkProperty.h>
#include<vtkDataSetMapper.h>
#include<vtkUnstructuredGrid.h>
#include<vtkMultiBlockDataSet.h>
#include<vtkPointData.h>
#include<vtkDataArray.h>
#include<vtkFloatArray.h>
#include<vtkMaskPoints.h>
#include<vtkGlyph3D.h>

#include <vtkMaskPoints.h>
#include "postFilterGlyph.h"

using namespace std;

postFilterGlyph::postFilterGlyph()
{
    m_vec_actor = vtkSmartPointer<vtkActor>::New();
    m_mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    m_lut = vtkSmartPointer<vtkLookupTable>::New();
    m_scalarBar = vtkSmartPointer<vtkScalarBarActor>::New();
    m_ungrid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    m_vectors_disp = vtkSmartPointer<vtkFloatArray>::New();
    m_vectors_s = vtkSmartPointer<vtkFloatArray>::New();
    m_arrow = vtkSmartPointer<vtkArrowSource>::New();
    
    m_mask = vtkSmartPointer<vtkMaskPoints>::New();
    m_glyph = vtkSmartPointer<vtkGlyph3D>::New();
    
    m_vectors_disp->SetNumberOfComponents(3);
    m_vectors_disp->SetName("disp_vector");
    
    m_vectors_s->SetNumberOfComponents(3);
    m_vectors_s->SetName("s_vector");
    
    //m_show_array_id = 0;
    //m_type_id = 0;
    //m_vec_array_name.clear();
    
    m_name1 = "disp_vector";
    m_name2 = "s_vector";
    m_max_scale = 10;
    m_show_scale = 3;
    
    m_mask_interval = 5;
    m_vector_type = 0;
    
    this->buildLut();
    this->buildScalarBar();
}

postFilterGlyph::~postFilterGlyph()
{

}

void postFilterGlyph::buildLut()
{
    // 创建一个色度图
    //vtkSmartPointer<vtkLookupTable> lut = vtkSmartPointer<vtkLookupTable>::New();
    m_lut->SetTableRange(-999, 999);
    m_lut->SetHueRange(0.67, 0.0);
    m_lut->Build();
    return;
}

void postFilterGlyph::buildScalarBar()
{
    m_scalarBar->SetLookupTable(m_lut);
    m_scalarBar->SetTitle("CSRock");
    m_scalarBar->SetNumberOfLabels(5);
    m_scalarBar->SetPosition(0.9, 0.2);
    m_scalarBar->SetWidth(0.1);
    m_scalarBar->SetHeight(0.6);
    return;
}   

void postFilterGlyph::setUnstructuredGrid(vtkSmartPointer<vtkUnstructuredGrid> ungrid)
{
     m_ungrid->DeepCopy(ungrid);
    //m_ungrid->DeepCopy(ungrid);
    return;
}

void postFilterGlyph::calMaxScale()
{
    if(!m_ungrid)
        return ;
    vtkBoundingBox* bound_tmp = new vtkBoundingBox;
    bound_tmp->AddBounds(m_ungrid->GetBounds()); 
    m_max_scale = bound_tmp->GetMaxLength()*0.1;
}

void postFilterGlyph::calVecType(std::vector<std::string>& m_vec_array_name)
{
    int nNumScalar = 3;
    //m_ungrid->GetNumberOfPointArrays();
    int count1 =0;
    int count2 =0;
    for(int i =0;i< m_vec_array_name.size();i++)
    {
        std::string ps =  m_vec_array_name[i];
        //std::cout<<ps<<std::endl;
        //reader->GetPointArrayName(i);
        if(ps == "SX" || ps == "SY" || ps == "SZ")
            count1++;
            
        if(ps == "DISP-X" || ps == "DISP-Y" || ps == "DISP-Z")
            count2++;
    }
    if(count1==3 && count2<3)
    { 
        m_vector_type = 1;
    }
    else if(count1<3 && count2==3)
    {
        m_vector_type = 2;
    }
    else if(count1==3 && count2==3)
    {
        m_vector_type = 3;
    }
    else 
    {
        m_vector_type = 0;
    }
    return;
}

void postFilterGlyph::initProcess()
{   
    if(m_vector_type == 1 || m_vector_type == 3)
    {
        vtkSmartPointer<vtkFloatArray> datax = vtkSmartPointer<vtkFloatArray>::New();
        datax->DeepCopy(m_ungrid->GetPointData()->GetArray("DISP-X"));
        vtkSmartPointer<vtkFloatArray> datay = vtkSmartPointer<vtkFloatArray>::New();
        datay->DeepCopy(m_ungrid->GetPointData()->GetArray("DISP-Y"));
        vtkSmartPointer<vtkFloatArray> dataz = vtkSmartPointer<vtkFloatArray>::New();
        dataz->DeepCopy(m_ungrid->GetPointData()->GetArray("DISP-Z"));

        m_vectors_disp->Resize(datax->GetNumberOfTuples());
        m_vectors_disp->SetNumberOfTuples(datax->GetNumberOfTuples());
        
        for (vtkIdType i = 0;i < datax->GetNumberOfTuples();i++)
        {
            double vecxyz[3] = { datax->GetTuple1(i),datay->GetTuple1(i),dataz->GetTuple1(i) };
            m_vectors_disp->SetTuple(i,vecxyz);
        }
    }
    
    if(m_vector_type == 2 || m_vector_type == 3)
    {
        vtkSmartPointer<vtkFloatArray> datax = vtkSmartPointer<vtkFloatArray>::New();
        datax->DeepCopy(m_ungrid->GetPointData()->GetArray("S1"));
        vtkSmartPointer<vtkFloatArray> datay = vtkSmartPointer<vtkFloatArray>::New();
        datay->DeepCopy(m_ungrid->GetPointData()->GetArray("S2"));
        vtkSmartPointer<vtkFloatArray> dataz = vtkSmartPointer<vtkFloatArray>::New();
        dataz->DeepCopy(m_ungrid->GetPointData()->GetArray("S3"));
        
        m_vectors_s->Resize(datax->GetNumberOfTuples());
        m_vectors_s->SetNumberOfTuples(datax->GetNumberOfTuples());
        
        for (vtkIdType i = 0;i < datax->GetNumberOfTuples();i++)
        {
            double vecxyz[3] = { datax->GetTuple1(i),datay->GetTuple1(i),dataz->GetTuple1(i) };
            m_vectors_s->SetTuple(i,vecxyz);
        }
    }
    else
    {
        return;
    }
}

void postFilterGlyph::showProcess(std::string show_name)
{
    if(show_name=="disp_vector")
        m_ungrid->GetPointData()->SetVectors(m_vectors_disp);
    else if(show_name=="s_vector")
        m_ungrid->GetPointData()->SetVectors(m_vectors_s);
    
    //data_use->GetPointData()->SetVectors(vectors);
    m_ungrid->GetPointData()->Update();
   
    m_mask->SetInputData(m_ungrid);
    m_mask->SetOnRatio(m_mask_interval);
    //mask->SetOnRatio(2);
    m_mask->RandomModeOff();
    m_mask->Update();
    
    //生成箭头符号
    m_arrow->Update();
    vtkSmartPointer<vtkGlyph3D>glyph = vtkSmartPointer<vtkGlyph3D>::New();
    m_glyph->SetInputData(m_mask->GetOutput());
    m_glyph->SetSourceData(m_arrow->GetOutput());
    //glyph->SetVectorModeToUseNormal();
    m_glyph->SetColorModeToColorByVector();

    m_glyph->SetScaleFactor(m_show_scale);
    m_glyph->Update(); 
     
    double* range = m_glyph->GetOutput()->GetPointData()->GetArray("GlyphVector")->GetRange();
    m_mapper->SetInputData(m_glyph->GetOutput());
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SelectColorArray("GlyphVector");
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range);
    m_mapper->SetColorModeToMapScalars();

    m_vec_actor->SetMapper(m_mapper);
    m_vec_actor->GetProperty()->SetRepresentationToSurface();
    return;
}

void postFilterGlyph::setShowScale(float show_scale)
{
    m_show_scale = show_scale;
}

void postFilterGlyph::setMaskInterval(int mask_interval)
{
    m_mask_interval = mask_interval;
}

void postFilterGlyph::updateShowScale(float show_scale)
{
    m_glyph->SetScaleFactor(show_scale);
    m_glyph->Update(); 
    return;
}

void postFilterGlyph::updateMaskInterval(int mask_interval)
{
    m_mask->SetOnRatio(mask_interval);
    m_mask->Update(); 
    return;
}

void postFilterGlyph::update(float show_scale,int mask_interval)
{
    updateMaskInterval(mask_interval); 
    updateShowScale(show_scale); 
    return;
}

vtkSmartPointer<vtkActor> postFilterGlyph::getActor()
{
    return m_vec_actor;
}

vtkSmartPointer<vtkLookupTable> postFilterGlyph::getLut()
{
    return m_lut;
}














