#pragma once

#include <vtkActor.h>
#include <vtkDataSetMapper.h>
#include <vtkLookupTable.h>
#include <vtkScalarBarActor.h>
#include <vtkFloatArray.h>
#include <vector>

#include <vtkTableBasedClipDataSet.h>
#include<vtkCutter.h>

class postFilterSlice
{
public:
    postFilterSlice();
    ~postFilterSlice();
    
private:
    vtkSmartPointer<vtkActor> m_slice_actor;
    vtkSmartPointer<vtkActor> m_slice_main_actor;
    vtkSmartPointer<vtkDataSetMapper> m_mapper;
    vtkSmartPointer<vtkDataSetMapper> m_mapper2;
    vtkSmartPointer<vtkLookupTable> m_lut;
    vtkSmartPointer<vtkUnstructuredGrid> m_ungrid;
    vtkSmartPointer<vtkUnstructuredGrid> m_ungrid2;
    vtkSmartPointer<vtkScalarBarActor> m_scalarBar;
     
private:
    void buildScalarBar();
    void buildLut();
    
    int m_cell_type;
    float m_bound_value[6];
    float m_x_range[2];
    float m_y_range[2];
    float m_z_range[2];
    
    float m_point_opacity;
    int m_point_size;
    float m_point_color_x;
    float m_point_color_y;
    float m_point_color_z;
    
public:
    vtkSmartPointer<vtkActor> getActor();
    vtkSmartPointer<vtkActor> getSliceMainActor();
    vtkSmartPointer<vtkLookupTable> getLut();
    
    void setUnstructuredGrid(vtkSmartPointer<vtkUnstructuredGrid> ungrid);
    void showProcess(int current_id,int use_point = 0);
    void showProcessMulti(int current_id);
    void showProcessSingle(int current_id);
    void initProcess(int cell_type=0);
    
    void showMainProcess(int current_id,int plane_id,float actual_value);
    
    void setCellType(int cell_type);
    int getCellType()const{return m_cell_type;}
    
    float *getRangeX();
    float *getRangeY();
    float *getRangeZ();
       
    void updateUsePointData(int current_id,int plane_id,float actua_value,bool b_invert_value);
    void updateUseCellData(int current_id,int plane_id,float actua_value,bool b_invert_value);
    void update(int current_id,int plane_id,float actua_value,bool b_invert_value);
    
    //void updateMulti(int current_id,float contour_min_value,float contour_max_value,int contour_num);
    //void updateSingle(int current_id,float contour_value);
    
    void updateSliceMulti(int current_id,int plane_id,float min_value,float max_value,int i_num);
    void updateSliceMultiPoint(int current_id,int plane_id,float min_value,float max_value,int i_num);
    void updateSliceMultiCell(int current_id,int plane_id,float min_value,float max_value,int i_num);
    
    void updateSliceSingle(int current_id,int plane_id,float actual_value);
    void updateSliceSinglePoint(int current_id,int plane_id,float actual_value);
    void updateSliceSingleCell(int current_id,int plane_id,float actual_value);
    
    void updatePointColor(float color_x, float color_y, float color_z);
    void updatePointOpacity(float opacity);
    void updatePointSize(int point_size);

    void setPointOpacity(float opacity);
    void setPointSize(int point_size);
    void setPointColor(float color_x,float color_y,float color_z);
};
















