#include<iostream>

#include<vtkProperty.h>
#include<vtkDataSetMapper.h>
#include<vtkUnstructuredGrid.h>
#include<vtkMultiBlockDataSet.h>
#include<vtkPointData.h>
#include<vtkCellData.h>
#include<vtkDataArray.h>
#include<vtkFloatArray.h>
#include<vtkContourFilter.h>
#include<vtkThreshold.h>

#include<vtkTecplotReader.h>

#include "postFilterThreshold.h"

using namespace std;

postFilterThreshold::postFilterThreshold()
{
    m_thr_actor = vtkSmartPointer<vtkActor>::New();
    m_mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    m_lut = vtkSmartPointer<vtkLookupTable>::New();
    m_scalarBar = vtkSmartPointer<vtkScalarBarActor>::New();
    m_ungrid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    m_cell_type = 0;
    
    this->buildLut();
    this->buildScalarBar();
}

postFilterThreshold::~postFilterThreshold()
{

}
void postFilterThreshold::setUnstructuredGrid(vtkSmartPointer<vtkUnstructuredGrid> ungrid)
{
    m_ungrid->DeepCopy(ungrid);
    return;
}

void postFilterThreshold::showProcess(int current_id)
{
    if(m_cell_type==0)
    {
        int m_show_array_id = current_id;
        vtkSmartPointer<vtkThreshold> thre = vtkSmartPointer<vtkThreshold>::New();
        m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
        vtkDataArray* array_scalar = m_ungrid->GetPointData()->GetArray(m_show_array_id);
        double* range = array_scalar->GetRange();
        thre->SetInputData(m_ungrid);
        //thre->SetThresholdFunction(1);
        //thre->SetUpperThreshold(value_upper);  
        //thre->SetLowerThreshold(value_lower);
        //thre->InvertOff();
        thre->Update();
   
        //double* range = m_ungrid->GetPointData()->GetArray(m_show_array_id)->GetRange();
    
        m_mapper->SetInputData(thre->GetOutput());
        m_mapper->SetLookupTable(m_lut);
        m_mapper->SetScalarModeToUsePointFieldData();
        m_mapper->SelectColorArray(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
        m_mapper->SetScalarRange(range); 

        m_thr_actor->SetMapper(m_mapper);
    }
    else if(m_cell_type==1)
    {
        int m_show_array_id = current_id;
        vtkSmartPointer<vtkThreshold> thre = vtkSmartPointer<vtkThreshold>::New();
        m_ungrid->GetCellData()->SetActiveScalars(m_ungrid->GetCellData()->GetArrayName(m_show_array_id));
        vtkDataArray* array_scalar = m_ungrid->GetCellData()->GetArray(m_show_array_id);
        double* range = array_scalar->GetRange();
        thre->SetInputData(m_ungrid);
        //thre->SetThresholdFunction(1);
        //thre->SetUpperThreshold(value_upper);  
        //thre->SetLowerThreshold(value_lower);
        //thre->InvertOff();
        thre->Update();
   
        //double* range = m_ungrid->GetCelltData()->GetArray(m_show_array_id)->GetRange();
    
        m_mapper->SetInputData(thre->GetOutput());
        m_mapper->SetLookupTable(m_lut);
        m_mapper->SetScalarModeToUseCellFieldData();
        m_mapper->SelectColorArray(m_ungrid->GetCellData()->GetArrayName(m_show_array_id));
        m_mapper->SetScalarRange(range); 

        m_thr_actor->SetMapper(m_mapper);
    }
    return;
}

void postFilterThreshold::update(int current_id,float actua_value_min,float actua_value_max,bool b_invert_value)
{
    if(m_cell_type==0)
    {
        updateUsePointData(current_id,actua_value_min,actua_value_max,b_invert_value);
    }
    else if(m_cell_type==1)
    {
        updateUseCellData(current_id,actua_value_min,actua_value_max,b_invert_value); 
    }
    return;
}

void postFilterThreshold::updateUsePointData(int current_id,float actua_value_min,float actua_value_max,bool b_invert_value)
{
    int show_array_id = current_id;
    m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(show_array_id));
    vtkDataArray* array_scalar = m_ungrid->GetPointData()->GetArray(show_array_id);
    double* range = array_scalar->GetRange();
    
    float value_upper = actua_value_max;
    float value_lower = actua_value_min;
    
    vtkSmartPointer<vtkThreshold> thre = vtkSmartPointer<vtkThreshold>::New();
    thre->SetInputData(m_ungrid);
    thre->SetThresholdFunction(1);
    thre->SetUpperThreshold(value_upper);  
    thre->SetLowerThreshold(value_lower);
    thre->SetInvert(b_invert_value);
    //thre->InvertOff();
    thre->Update();

    //m_mapper->SetInputData(thre->GetOutput());
    m_mapper->SetInputConnection(thre->GetOutputPort());
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SelectColorArray(show_array_id);
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range); 
    m_thr_actor->SetMapper(m_mapper);
    m_thr_actor->GetProperty()->SetRepresentationToSurface();

    return;
}

void postFilterThreshold::updateUseCellData(int current_id,float actua_value_min,float actua_value_max,bool b_invert_value)
{
    int show_array_id = current_id;
    m_ungrid->GetCellData()->SetActiveScalars(m_ungrid->GetCellData()->GetArrayName(show_array_id));
    vtkDataArray* array_scalar = m_ungrid->GetCellData()->GetArray(show_array_id);
    double* range = array_scalar->GetRange();
    
    float value_upper = actua_value_max;
    float value_lower = actua_value_min;
    
    vtkSmartPointer<vtkThreshold> thre = vtkSmartPointer<vtkThreshold>::New();
    thre->SetInputData(m_ungrid);
    thre->SetThresholdFunction(1);
    thre->SetUpperThreshold(value_upper);  
    thre->SetLowerThreshold(value_lower);
    thre->SetInvert(b_invert_value);
    //thre->InvertOff();
    thre->Update();

    //m_mapper->SetInputData(thre->GetOutput());
    m_mapper->SetInputConnection(thre->GetOutputPort());
    m_mapper->SetLookupTable(m_lut);
    m_mapper->SetScalarModeToUseCellFieldData();
    m_mapper->SelectColorArray(show_array_id);
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range); 
    m_thr_actor->SetMapper(m_mapper);
    m_thr_actor->GetProperty()->SetRepresentationToSurface();

    return;
}

void postFilterThreshold::setCellType(int cell_type)
{
    m_cell_type = cell_type;
    return;
}

void postFilterThreshold::initProcess(int cell_type)
{
    setCellType(cell_type);
    return;
}

void postFilterThreshold::buildLut()
{
    // 创建一个色度图
    //vtkSmartPointer<vtkLookupTable> lut = vtkSmartPointer<vtkLookupTable>::New();
    m_lut->SetTableRange(-999, 999);
    m_lut->SetHueRange(0.67, 0.0);
    m_lut->Build();
    return;
}

void postFilterThreshold::buildScalarBar()
{
    m_scalarBar->SetLookupTable(m_lut);
    m_scalarBar->SetTitle("CSRock");
    m_scalarBar->SetNumberOfLabels(5);
    m_scalarBar->SetPosition(0.9, 0.2);
    m_scalarBar->SetWidth(0.1);
    m_scalarBar->SetHeight(0.6);
    return;
}

vtkSmartPointer<vtkActor> postFilterThreshold::getActor()
{
    return m_thr_actor;
}

vtkSmartPointer<vtkLookupTable> postFilterThreshold::getLut()
{
    return m_lut;
}














