#include<iostream>

#include<vtkProperty.h>
#include<vtkDataSetMapper.h>
#include<vtkXMLUnstructuredGridReader.h>
#include<vtkUnstructuredGrid.h>
#include<vtkTecplotReader.h>
#include<vtkMultiBlockDataSet.h>
#include<vtkPointData.h>
#include<vtkDataArray.h>

#include "postShowVtu.h"

using namespace std;

postShowVtu::postShowVtu()
{
    m_vtu_actor = vtkSmartPointer<vtkActor>::New();
    m_mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    m_lut = vtkSmartPointer<vtkLookupTable>::New();
    m_scalarBar = vtkSmartPointer<vtkScalarBarActor>::New();
    m_ungrid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    m_ungrid2 = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    m_color_x = 0.75;
    m_color_y = 0.75;
    m_color_z = 0.75;
    m_opacity = 1.0;
    m_specular = 0.0;
    //m_b_edge_on = true;
    
    m_point_size = 1.0;
    m_line_width = 1.0;
    m_edge_color_x = 0.0;
    m_edge_color_y = 0.0;
    m_edge_color_z = 0.0;
    
    m_show_array_id = 0;
    m_type_id = 0;
    m_vec_array_name.clear();
    m_vec_min_value.clear();
    m_vec_max_value.clear();
    this->buildLut();
    this->buildScalarBar();
}

postShowVtu::~postShowVtu()
{

}

void postShowVtu::buildLut()
{
    // 创建一个色度图
    //vtkSmartPointer<vtkLookupTable> lut = vtkSmartPointer<vtkLookupTable>::New();
    m_lut->SetTableRange(-100, 100); // 使用数据的最低温和最高温
    m_lut->SetHueRange(0.67, 0.0); // 0.67 对应蓝色, 0.0 对应红色
    m_lut->Build();
    return;
}

void postShowVtu::buildScalarBar()
{
    m_scalarBar->SetLookupTable(m_lut);
    m_scalarBar->SetTitle("CSRock");
    m_scalarBar->SetNumberOfLabels(5);
    m_scalarBar->SetPosition(0.9, 0.2);
    m_scalarBar->SetWidth(0.1);
    m_scalarBar->SetHeight(0.6);
    return;
}   

void postShowVtu::readShowResultVtuFile(std::string file_name)
{
    m_type_id = 0;
    vtkSmartPointer<vtkXMLUnstructuredGridReader>reader = vtkSmartPointer<vtkXMLUnstructuredGridReader>::New();
    reader->SetFileName(file_name.c_str());
    reader->Update();

    m_ungrid = reader->GetOutput();
    m_ungrid2->DeepCopy(m_ungrid);
    m_mapper->SetInputData(m_ungrid);
    
    //std::cout<<reader->GetNumberOfDataArrays()<<std::endl;
    int nNumScalar = reader->GetNumberOfPointArrays();
    m_vec_array_name.clear();
    m_vec_min_value.clear();
    m_vec_max_value.clear();
    
    //auto vtkSmartPointer<vtkDataArray> scalars = vtkSmartPointer<vtkDataArray>::New();
    for(int i =0;i< nNumScalar;i++)
    {
        std::string ps = reader->GetPointArrayName(i);
        m_vec_array_name.push_back(ps);
        //std::cout<<ps<<std::endl;
        
        auto scalars = m_ungrid->GetPointData()->GetArray(i);
        m_vec_min_value.push_back(scalars->GetRange()[0]);
        m_vec_max_value.push_back(scalars->GetRange()[1]);
    }
    
    auto scalars = m_ungrid->GetPointData()->GetArray(m_show_array_id);
    double* range = scalars->GetRange();
    
    //double minTemp = scalars->GetRange()[0];
    //double maxTemp = scalars->GetRange()[1];
    //std::cout<<scalars->GetRange()[0]<<std::endl;
    //std::cout<<scalars->GetRange()[1]<<std::endl;

    m_mapper->SetInputData(m_ungrid);
    m_mapper->SetLookupTable(m_lut); 
    
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range);
    m_mapper->SelectColorArray(m_show_array_id);
  
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range);
    return;
    
}

void postShowVtu::readShowResultTecFile(std::string file_name)
{
    m_type_id = 0;
    vtkSmartPointer<vtkTecplotReader> reader = vtkSmartPointer<vtkTecplotReader>::New();
    reader->SetFileName(file_name.c_str());
    reader->Update();
    
    vtkSmartPointer<vtkMultiBlockDataSet> multi = vtkSmartPointer<vtkMultiBlockDataSet>::New();
    multi = reader->GetOutput();
    
    m_ungrid = (vtkUnstructuredGrid*)(multi->GetBlock(0));
    m_ungrid2->DeepCopy(m_ungrid);
    m_mapper->SetInputData(m_ungrid);
    int nNumScalar = reader->GetNumberOfDataArrays();
    
    m_vec_array_name.clear();
    m_vec_min_value.clear();
    m_vec_max_value.clear();
    
    //vtkSmartPointer<vtkDataArray> scalars = vtkSmartPointer<vtkDataArray>::New();
    for(int i =0;i< nNumScalar;i++)
    {
        std::string ps = reader->GetDataArrayName(i);
        m_vec_array_name.push_back(ps);
        //std::cout<<ps<<std::endl;
        
        auto scalars = m_ungrid->GetPointData()->GetArray(i);
        m_vec_min_value.push_back(scalars->GetRange()[0]);
        m_vec_max_value.push_back(scalars->GetRange()[1]);
    }

    auto scalars = m_ungrid->GetPointData()->GetArray(m_show_array_id);
   
    double* range = scalars->GetRange();
    //double minTemp = scalars->GetRange()[0];
    //double maxTemp = scalars->GetRange()[1];
    //std::cout<<scalars->GetRange()[0]<<std::endl;
    //std::cout<<scalars->GetRange()[1]<<std::endl;

    m_mapper->SetInputData(m_ungrid);
    m_mapper->SetLookupTable(m_lut); 
    
    m_mapper->SetScalarModeToUsePointFieldData();
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range);
   
    m_mapper->SelectColorArray(m_show_array_id);
  
    m_mapper->SetUseLookupTableScalarRange(false);
    m_mapper->SetScalarRange(range);
    return;
    
    //m_mapper->SelectColorArray(m_ungrid->GetPointData()->GetArrayName(6));
    //m_mapper->UseLookupTableScalarRangeOff();
    //m_mapper->ScalarVisibilityOn();
    //m_mapper->SetArrayName(m_ungrid->GetPointData()->GetArrayName(6));
    //m_mapper->SetArrayId(12);
    //mapper->SetScaleModeToScaleByMagnitude();
    //mapper->SetScaleModeToScaleByVectorComponents();
    //mapper->SetScaleArray("ceshi");
}

void postShowVtu::showProcess()
{
    m_vtu_actor->SetMapper(m_mapper);
    //m_vtu_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    m_vtu_actor->GetProperty()->SetOpacity(m_opacity);
    
    m_vtu_actor->GetProperty()->SetAmbient(0.6);
    m_vtu_actor->GetProperty()->SetDiffuse(0.0);
    m_vtu_actor->GetProperty()->SetSpecular(0.0);

    changeTypeToSurface();
    return;
    //m_vtu_actor->GetProperty()->SetRepresentationToSurface();
    //m_vtu_actor->GetProperty()->SetEdgeVisibility(false);
}


void postShowVtu::changeScalar(int scalar_id)
{
   m_show_array_id = scalar_id;
   m_ungrid->GetPointData()->SetActiveScalars(m_ungrid->GetPointData()->GetArrayName(m_show_array_id));
   vtkDataArray* scalars = m_ungrid->GetPointData()->GetArray(m_show_array_id);
   double* range = scalars->GetRange();
   
   m_mapper->SetScalarRange(range);
   m_mapper->SelectColorArray(m_show_array_id);
   return;
   
   //std::cout<<range[0]<<std::endl;
   //std::cout<<range[1]<<std::endl;
   //m_mapper->SelectColorArray(data_use->GetPointData()->GetArrayName(i));
   //m_scalarBarWgt->GetScalarBarActor()->SetLookupTable(m_Actor->GetMapper()->GetLookupTable());
}


void postShowVtu::changeTypeToSurface()
{
    m_vtu_actor->GetProperty()->SetRepresentationToSurface();
    m_vtu_actor->GetProperty()->SetEdgeVisibility(false);
    return;
}

void postShowVtu::changeTypeToSurfaceWithEdge()
{
    m_vtu_actor->GetProperty()->SetRepresentationToSurface();
    m_vtu_actor->GetProperty()->SetEdgeVisibility(true);
    return;
}

void postShowVtu::changeTypeToPoint()
{
    m_vtu_actor->GetProperty()->SetRepresentationToPoints();
    return;
}

void postShowVtu::changeTypeToWireframe()
{
    m_vtu_actor->GetProperty()->SetRepresentationToWireframe();
    return;
}

void postShowVtu::changeType(int type_id)
{
    //std::cout<<type_id<<std::endl;
    
    if (type_id==1)
    {
        changeTypeToPoint();
    }
    else if(type_id==2)
    {
        changeTypeToWireframe();
    }
    else if(type_id==3)
    {
        changeTypeToSurfaceWithEdge();
    }
    else
    {
        changeTypeToSurface();
    }
    return;
}

vtkSmartPointer<vtkLookupTable> postShowVtu::getLut()
{
    return m_lut;
}

vtkSmartPointer<vtkActor> postShowVtu::getActor()
{
    return m_vtu_actor;
}

vtkSmartPointer<vtkScalarBarActor> postShowVtu::getScalarBar()
{
    return m_scalarBar;
}

vtkSmartPointer<vtkUnstructuredGrid> postShowVtu::getUnstructuredGrid()
{
    return m_ungrid;
}

vtkSmartPointer<vtkUnstructuredGrid> postShowVtu::getUnstructuredGrid2()
{
    return m_ungrid2;
}

void postShowVtu::setPointSize(float point_size)
{
    m_point_size = point_size;
    return;
}

void postShowVtu::setLineWidth(float line_width)
{
    m_line_width = line_width;
    return;
}

void postShowVtu::setEdgeColor(float color_x, float color_y,float color_z)
{
    m_edge_color_x = color_x;
    m_edge_color_y = color_y;
    m_edge_color_z = color_z;
    return;
}

void postShowVtu::setColor(float color_x, float color_y, float color_z)
{
    m_color_x = color_x;
    m_color_y = color_y;
    m_color_z = color_z;
    return;
}

void postShowVtu::setOpacity(float opacity)
{
    m_opacity =  opacity;
    return;
}

void postShowVtu::setSpecular(float specular)
{
    m_specular =  specular;
    return;
}

//void postShowVtu::setStatusEdge(bool b_edge_on)
//{
    //m_b_edge_on = b_edge_on;
//}

void postShowVtu::update()
{  
    updateMainProperty();
}

void postShowVtu::updateMainProperty()
{  
   
    m_vtu_actor->GetProperty()->SetOpacity(m_opacity);
    m_vtu_actor->GetProperty()->SetSpecular(m_specular);
    
    m_vtu_actor->GetProperty()->SetPointSize(m_point_size);
    m_vtu_actor->GetProperty()->SetEdgeColor(m_edge_color_x, m_edge_color_y, m_edge_color_z);
    m_vtu_actor->GetProperty()->SetLineWidth(m_line_width);
    
    //m_vtu_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    //m_vtu_actor->GetProperty()->SetEdgeVisibility(m_b_edge_on);
}

void postShowVtu::updatePointSize(float point_size)
{
    m_vtu_actor->GetProperty()->SetPointSize(point_size);
    return;
}

void postShowVtu::updateLineWidth(float line_width)
{
    m_vtu_actor->GetProperty()->SetLineWidth(line_width);
    return;
}

void postShowVtu::updateEdgeColor(float color_x, float color_y,float color_z)
{
    m_vtu_actor->GetProperty()->SetEdgeColor(color_x, color_y, color_z);
    return;
}

void postShowVtu::updateColor(float color_x, float color_y, float color_z)
{
    m_vtu_actor->GetProperty()->SetColor(color_x, color_y, color_z);
    return;
}

void postShowVtu::updateOpacity(float opacity)
{
    m_vtu_actor->GetProperty()->SetOpacity(opacity);
    return;
}

void postShowVtu::updateSpecular(float specular)
{
    m_vtu_actor->GetProperty()->SetSpecular(specular);
    return;
}

//void postShowVtu::updateStatusEdge(bool b_edge_on)
//{
   //m_vtu_actor->GetProperty()->SetEdgeVisibility(m_b_edge_on);
//}

















