#pragma once

#include <vtkActor.h>
#include <vtkDataSetMapper.h>
#include <vtkLookupTable.h>
#include <vtkScalarBarActor.h>

#include <vector>

class postShowVtu
{
public:
    postShowVtu();
    ~postShowVtu();
    
private:
    vtkSmartPointer<vtkActor> m_vtu_actor;
    vtkSmartPointer<vtkDataSetMapper> m_mapper;
    vtkSmartPointer<vtkLookupTable> m_lut;
    vtkSmartPointer<vtkScalarBarActor> m_scalarBar;
    vtkSmartPointer<vtkUnstructuredGrid> m_ungrid;
    vtkSmartPointer<vtkUnstructuredGrid> m_ungrid2;
    
    float m_color_x;
    float m_color_y;
    float m_color_z;
    float m_opacity;
    float m_specular;
    float m_point_size;
    float m_line_width;
    float m_edge_color_x;
    float m_edge_color_y;
    float m_edge_color_z;
    //bool m_b_edge_on;
    
    int m_show_array_id;
    int m_type_id;
    std::vector<std::string> m_vec_array_name;
    std::vector<float> m_vec_min_value;
    std::vector<float> m_vec_max_value;
    
private:
    void changeTypeToSurface();
    void changeTypeToSurfaceWithEdge();
    void changeTypeToPoint();
    void changeTypeToWireframe();

public:
    vtkSmartPointer<vtkActor> getActor();
    vtkSmartPointer<vtkLookupTable> getLut();
    vtkSmartPointer<vtkScalarBarActor> getScalarBar();
    vtkSmartPointer<vtkUnstructuredGrid> getUnstructuredGrid();
    vtkSmartPointer<vtkUnstructuredGrid> getUnstructuredGrid2();
    std::vector<std::string>& getScalarName(){return m_vec_array_name;}
    std::vector<float>& getVecMinValue(){return m_vec_min_value;}
    std::vector<float>& getVecMaxValue(){return m_vec_max_value;}

    void readShowResultVtuFile(std::string file_name);
    void readShowResultTecFile(std::string file_name);
    
    void changeScalar(int scalar_id);
    void changeType(int type_id);
    
    void buildScalarBar();
    void buildLut();
      
    void showProcess();
    
    void setColor(float color_x, float color_y, float color_z);
    void setOpacity(float opacity);
    void setSpecular(float specular);
    
    void setPointSize(float point_size);
    void setLineWidth(float line_width);
    void setEdgeColor(float color_x,float color_y,float color_z);
    
    //void setStatusEdge(bool b_edge_on);
    
    float getOpacity()const{return m_opacity;}
    float getSpecular()const{return m_specular;}
    
    float getPointSize()const{return m_point_size;}
    float getLineWidth()const{return m_line_width;}

    //bool getStatusEdge()const{return m_b_edge_on;}

    void update();
    void updateMainProperty();
    void updateColor(float color_x, float color_y, float color_z);
    void updateOpacity(float opacity);
    void updateSpecular(float specular);
    
    void updatePointSize(float point_size);
    void updateLineWidth(float line_width);
    void updateEdgeColor(float color_x,float color_y,float color_z);
    
    //void updateStatusEdge(bool b_edge_on);
};
















