#include<iostream>

#include<vtkProperty.h>
#include<vtkDataSetMapper.h>
#include<vtkPolyData.h>
#include<vtkSTLWriter.h>
#include<vtkQuad.h>

#include "gen3DModel.h"
#include "readJson.h"

using namespace std;

gen3DModel::gen3DModel()
{
    m_stl_actor = vtkSmartPointer<vtkActor>::New();
    m_poly = vtkSmartPointer<vtkPolyData>::New();
    
    m_color_x = 0.75;
    m_color_y = 0.75;
    m_color_z = 0.75;
    m_opacity = 1.0;
    //m_b_edge_on = true;
    
    //1 center 0 leftBottom 
    m_offset_type = 0;
    m_length = 10;
    m_width = 10;
    m_height = 10;
    m_offset_x = 0;
    m_offset_y = 0;
    m_offset_z = 0;
    
    m_write_file_type = 1;   //2 binary  1 ascii
}

gen3DModel::~gen3DModel()
{

}

void gen3DModel::genProcess()
{
    float origin_x = m_offset_x;
    float origin_y = m_offset_y;
    float origin_z = m_offset_z;
   
    float length = m_length;
    float width = m_width;
    float height = m_height;
    
    float point_min_x = 0.0;
    float point_min_y = 0.0;
    float point_min_z = 0.0;
    float point_max_x = 0.0;
    float point_max_y = 0.0;
    float point_max_z = 0.0;
    
    int type = m_offset_type;
    if(0==type)
    {
        point_min_x = origin_x + 0;
        point_min_y = origin_y + 0;
        point_min_z = origin_y + 0;
        point_max_x = origin_x + length;
        point_max_y = origin_y + width;
        point_max_z = origin_z + height;
    }
    else if(1==type)
    {
        point_min_x = origin_x - length/2.0;
        point_min_y = origin_y - width/2.0;
        point_min_z = origin_y - height/2.0;
        point_max_x = origin_x + length/2.0;
        point_max_y = origin_y + width/2.0;
        point_max_z = origin_z + height/2.0;
   }
   
   vtkSmartPointer<vtkPoints> points = vtkSmartPointer<vtkPoints>::New();
   float point[3];
   point[0] = point_min_x;
   point[1] = point_min_y;
   point[2] = point_min_z;
   points->InsertNextPoint(point);
       
   point[0] = point_max_x;
   point[1] = point_min_y;
   point[2] = point_min_z;
   points->InsertNextPoint(point);
       
   point[0] = point_max_x;
   point[1] = point_max_y;
   point[2] = point_min_z;
   points->InsertNextPoint(point);
       
   point[0] = point_min_x;
   point[1] = point_max_y;
   point[2] = point_min_z;
   points->InsertNextPoint(point);
       
   point[0] = point_min_x;
   point[1] = point_min_y;
   point[2] = point_max_z;
   points->InsertNextPoint(point);
       
   point[0] = point_max_x;
   point[1] = point_min_y;
   point[2] = point_max_z;
   points->InsertNextPoint(point);
       
   point[0] = point_max_x;
   point[1] = point_max_y;
   point[2] = point_max_z;
   points->InsertNextPoint(point);
       
   point[0] = point_min_x;
   point[1] = point_max_y;
   point[2] = point_max_z;
   points->InsertNextPoint(point); 
        
   //创建vtk拓扑数据结构
   vtkSmartPointer<vtkCellArray> cells = vtkSmartPointer<vtkCellArray>::New();
   vtkSmartPointer<vtkQuad> quadpoly = vtkSmartPointer<vtkQuad>::New();
   //vtkSmartPointer<vtkPolyData> poly = vtkSmartPointer<vtkPolyData>::New();

   quadpoly->GetPointIds()->SetId(0, 0);
   quadpoly->GetPointIds()->SetId(1, 1);
   quadpoly->GetPointIds()->SetId(2, 2);
   quadpoly->GetPointIds()->SetId(3, 3);
   cells->InsertNextCell(quadpoly);
       
   quadpoly->GetPointIds()->SetId(0, 4);
   quadpoly->GetPointIds()->SetId(1, 5);
   quadpoly->GetPointIds()->SetId(2, 6);
   quadpoly->GetPointIds()->SetId(3, 7);
   cells->InsertNextCell(quadpoly);
       
   quadpoly->GetPointIds()->SetId(0, 0);
   quadpoly->GetPointIds()->SetId(1, 1);
   quadpoly->GetPointIds()->SetId(2, 5);
   quadpoly->GetPointIds()->SetId(3, 4);
   cells->InsertNextCell(quadpoly);
       
   quadpoly->GetPointIds()->SetId(0, 3);
   quadpoly->GetPointIds()->SetId(1, 2);
   quadpoly->GetPointIds()->SetId(2, 6);
   quadpoly->GetPointIds()->SetId(3, 7);
   cells->InsertNextCell(quadpoly);
       
   quadpoly->GetPointIds()->SetId(0, 3);
   quadpoly->GetPointIds()->SetId(1, 0);
   quadpoly->GetPointIds()->SetId(2, 4);
   quadpoly->GetPointIds()->SetId(3, 7);
   cells->InsertNextCell(quadpoly);
       
   quadpoly->GetPointIds()->SetId(0, 2);
   quadpoly->GetPointIds()->SetId(1, 1);
   quadpoly->GetPointIds()->SetId(2, 5);
   quadpoly->GetPointIds()->SetId(3, 6);
   cells->InsertNextCell(quadpoly);

   m_poly->SetPoints(points);
   m_poly->SetPolys(cells);
   
   vtkSmartPointer<vtkDataSetMapper>mapper = vtkSmartPointer<vtkDataSetMapper>::New();
   mapper->SetInputData(m_poly);
 
   m_stl_actor->SetMapper(mapper);
   m_stl_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
   m_stl_actor->GetProperty()->SetOpacity(m_opacity);
   m_stl_actor->GetProperty()->SetEdgeVisibility(0);
}

void gen3DModel::readJsonFile(string json_name)
{
    CasRockModel* struct_model = new CasRockModel;
    readJson *use_readJson = new readJson;
    use_readJson->readModelJson(json_name,struct_model);
    
    m_offset_type = struct_model->offset_type;
    m_length = struct_model->length;
    m_width = struct_model->width;
    m_height = 0;
    m_offset_x = struct_model->offset_x;
    m_offset_y = struct_model->offset_y;
    m_offset_z = 0;
    
    delete struct_model;
    delete use_readJson;
    struct_model = NULL;
    use_readJson = NULL;
}

void gen3DModel::writeJsonFile(string json_name)
{
    CasRockModel* struct_model = new CasRockModel;
    readJson *use_readJson = new readJson;
    struct_model->geo_type = 1;
    struct_model->offset_type = m_offset_type;
    struct_model->length = m_length;
    struct_model->width =  m_width;
    struct_model->height = m_height;
    struct_model->offset_x = m_offset_x;
    struct_model->offset_y = m_offset_y;
    struct_model->offset_z = m_offset_z;
  
    use_readJson->writeModelJson(json_name,struct_model);
    delete struct_model;
    delete use_readJson;
    struct_model = NULL;
    use_readJson = NULL;
}

void gen3DModel::setModelParams(CasRockModel* struct_model)
{
    m_offset_type = struct_model->offset_type;
    m_length = struct_model->length;
    m_width = struct_model->width;
    m_height = struct_model->height;
    m_offset_x = struct_model->offset_x;
    m_offset_y = struct_model->offset_y;
    m_offset_z = struct_model->offset_z;
}

void gen3DModel::getModelParams(CasRockModel *struct_model)
{
    struct_model->geo_type = 1;
    struct_model->offset_type = m_offset_type;
    struct_model->length = m_length;
    struct_model->width =  m_width;
    struct_model->height = m_height;
    struct_model->offset_x = m_offset_x;
    struct_model->offset_y = m_offset_y;
    struct_model->offset_z = m_offset_z;
}

void gen3DModel::writeStlFile(string stl_file_name)
{
    vtkSmartPointer<vtkSTLWriter> writer = vtkSmartPointer<vtkSTLWriter>::New();
    //writer->SetFileTypeToASCII();    //1
    //writer->SetFileTypeToBinary();     //2
    writer->SetFileType(m_write_file_type);
    //std::cout<<writer->GetFileType()<<std::endl;
    writer->SetFileName(stl_file_name.c_str());
    writer->SetInputData(m_poly);
    writer->Write();
}

vtkSmartPointer<vtkActor> gen3DModel::getActor()
{  
    return m_stl_actor;
}

void gen3DModel::setColor(float color_x, float color_y, float color_z)
{  
    m_color_x = color_x;
    m_color_y = color_y;
    m_color_z = color_z;
}

void gen3DModel::setOpacity(float opacity)
{  
    m_opacity =  opacity;
}

//void gen3DModel::setStatusEdge(bool b_edge_on)
//{  
//   m_b_edge_on = b_edge_on;
//}

void gen3DModel::update()
{  
    m_stl_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    m_stl_actor->GetProperty()->SetOpacity(m_opacity);
    //m_stl_actor->GetProperty()->SetEdgeVisibility(m_b_edge_on);
}

void gen3DModel::updateColor(float color_x, float color_y, float color_z)
{  
    m_stl_actor->GetProperty()->SetColor(color_x, color_y, color_z);
}

void gen3DModel::updateOpacity(float opacity)
{  
    m_stl_actor->GetProperty()->SetOpacity(opacity);
}

//void gen3DModel::updateStatusEdge(bool b_edge_on)
//{  
//   m_stl_actor->GetProperty()->SetEdgeVisibility(m_b_edge_on);
//}

void gen3DModel::setOffsetType(int offset_type)
{
     m_offset_type = offset_type;
}

void gen3DModel::setOffsetTypeToCenter()
{
    m_offset_type = 1;
}

void gen3DModel::setOffsetTypeToLeftBottom()
{
    m_offset_type = 0;
}

void gen3DModel::setOffsetX(float offset_x)
{
    m_offset_x = offset_x;
}
   
void gen3DModel::setOffsetY(float offset_y)
{
    m_offset_y = offset_y;
}
 
void gen3DModel::setOffsetZ(float offset_z)
{
    m_offset_z = offset_z;
}
     
void gen3DModel::setLength(float length)
{
    m_length = length;
}

void gen3DModel::setWidth(float width)
{
    m_width = width;
}

void gen3DModel::setHeight(float height)
{
    m_height = height;
}

void gen3DModel::setWriteFileMode(int write_file_type)
{
    m_write_file_type = write_file_type;
}

void gen3DModel::setWriteFileModeToAscii()
{
    m_write_file_type = 1;
}

void gen3DModel::setWriteFileModeToBinary()
{
    m_write_file_type = 2;
}

















