#include<iostream>
#include<fstream>
#include<string>
#include<sstream>
#include<vector> 

#include<vtkProperty.h>
#include<vtkActor.h>
#include<vtkDataSetMapper.h>
#include<vtkPoints.h>
#include<vtkCellData.h>
#include<vtkCellArray.h>
#include<vtkQuad.h>
#include<vtkHexahedron.h>
#include<vtkUnstructuredGrid.h>
#include<vtkUnstructuredGridWriter.h>
#include<vtkXMLUnstructuredGridWriter.h>

#include "readFlac.h"
using namespace std;

readFlac::readFlac()
{
    m_actor = vtkSmartPointer<vtkActor>::New();
    m_grid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    m_color_x = 0.75;
    m_color_y = 0.75;
    m_color_z = 0.75;
    m_opacity = 1.0;
    m_line_width = 1.0;
    m_write_xml_type = 1;    //1 vtu 0 vtk
    m_write_data_type = 1;   //1 binary  0 ascii
    
    m_cell_type = -1;
    
    vec_points.clear();
    vec_elements.clear();
}

readFlac::~readFlac()
{

}

bool readFlac::readFlacFile(string flac_file_name)
{
    vec_points.clear();
    vec_elements.clear();
    ifstream ifs;
    ifs.open(flac_file_name.c_str(), std::ios::in);
    if(!ifs.is_open() )
    {
        cerr << "Error: Cannot open the file '" << flac_file_name << "'." << endl;
        return false;  
    }
    string line;
    int count1 = 0;
    int count2 = 0;
    while(std::getline(ifs, line))
    {
        removeLineCarriage(&line);
        if(startWith(line, string("G")))
        {
            istringstream iss(line);
            string tem;
	    //跳过前两条
	    iss >> tem;
	    iss >> tem;
	    iss >> tem;
            //line_id = line.substr(8, 8);
            vec_points.push_back(atof(tem.c_str()));
            iss >> tem;
            vec_points.push_back(atof(tem.c_str()));
            iss >> tem;
            vec_points.push_back(atof(tem.c_str()));
        }
        
        if(startWith(line, string("Z")))
        {
            istringstream iss(line);
            string tem;
	    //跳过前两条
	    iss >> tem;
	    iss >> tem;
	    if(tem=="B8")
            {
                iss >> tem;
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                count1++;
            }
          
            if(tem=="T4")
            {
                iss >> tem;
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                iss >> tem;
                vec_elements.push_back(atoi(tem.c_str()));
                count2++;
            }
        }
    }
    if(count1>0)
    {
        m_cell_type = 1;
    }
    
    if(count2>0)
    {
        m_cell_type = 0;
    }
    ifs.close();
   
    return true;
}

void readFlac::processVtkStruct()
{
    vtkSmartPointer<vtkPoints> points = vtkSmartPointer<vtkPoints>::New();
    vtkSmartPointer<vtkCellArray> cells = vtkSmartPointer<vtkCellArray>::New();
    float point[3];
    for (auto iter = vec_points.begin(); iter != vec_points.end();)
    {
        //std::cout<<*iter++;
        //std::cout<<*iter++;
        //std::cout<<*iter++<<endl;
        point[0]=*iter++;
        point[1]=*iter++;
        point[2]=*iter++;
        points->InsertNextPoint(point);
    }

    m_grid->SetPoints(points);
    if(m_cell_type == 1)
    {
        float quad[8];
        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        {
            vtkSmartPointer<vtkHexahedron> hexpoly = vtkSmartPointer<vtkHexahedron>::New();
            quad[0]=*iter++;
            quad[1]=*iter++;
            quad[2]=*iter++;           
            quad[3]=*iter++;
            quad[4]=*iter++;
            quad[5]=*iter++;
            quad[6]=*iter++;
            quad[7]=*iter++;
            hexpoly->GetPointIds()->SetId(0, quad[0]-1);
            hexpoly->GetPointIds()->SetId(1, quad[1]-1);
            hexpoly->GetPointIds()->SetId(2, quad[4]-1);
            hexpoly->GetPointIds()->SetId(3, quad[2]-1);
            hexpoly->GetPointIds()->SetId(4, quad[3]-1);
            hexpoly->GetPointIds()->SetId(5, quad[6]-1);
            hexpoly->GetPointIds()->SetId(6, quad[7]-1);
            hexpoly->GetPointIds()->SetId(7, quad[5]-1);
            m_grid->InsertNextCell(hexpoly->GetCellType(), hexpoly->GetPointIds());
       }  
   }
    
   if(m_cell_type == 0)
   {
       float quad[4];
       for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
       {
           vtkSmartPointer<vtkQuad> quadpoly = vtkSmartPointer<vtkQuad>::New();
           quad[0]=*iter++;
           quad[1]=*iter++;
           quad[2]=*iter++;
           quad[3]=*iter++;
           quadpoly->GetPointIds()->SetId(0, quad[0]-1);
           quadpoly->GetPointIds()->SetId(1, quad[1]-1);
           quadpoly->GetPointIds()->SetId(2, quad[3]-1);
           quadpoly->GetPointIds()->SetId(3, quad[2]-1);
           m_grid->InsertNextCell(quadpoly->GetCellType(), quadpoly->GetPointIds());
       }
    }
    vtkSmartPointer<vtkDataSetMapper>mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    mapper->SetInputData(m_grid);
    //m_actor->GetProperty()->EdgeVisibilityOn();
    m_actor->SetMapper(mapper);
    m_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    m_actor->GetProperty()->SetOpacity(m_opacity);
    m_actor->GetProperty()->SetRepresentationToWireframe();
}

vtkSmartPointer<vtkActor> readFlac::getActor()
{  
    return m_actor;
}

void readFlac::writeGridFile(string nodes_file_name,string elements_file_name)
{
    writeNodesFile(nodes_file_name);
    writeElementsFile(elements_file_name);
}

void readFlac::writeNodesFile(string nodes_file_name)
{
    fstream os_coordinate;
    os_coordinate.open(nodes_file_name.c_str(), ios::out);
    ostringstream ostr_nodes; 
    os_coordinate << "    " << to_string(vec_points.size()/3) << endl;
    os_coordinate << endl;
    float point[3];
    int count = 0;
    for (auto iter = vec_points.begin(); iter != vec_points.end();)
    {
        point[0]=*iter++;
        point[1]=*iter++;
        point[2]=*iter++;
        ostr_nodes<<"      ";
        ostr_nodes<<count + 1<<"     ";
        ostr_nodes<<setiosflags(ios::fixed)<<setiosflags(ios::right)<<setprecision(8)<<point[0]<<"     ";
        ostr_nodes<<setiosflags(ios::fixed)<<setiosflags(ios::right)<<setprecision(8)<<point[1]<<"     ";;
        ostr_nodes<<setiosflags(ios::fixed)<<setiosflags(ios::right)<<setprecision(8)<<point[2]<<"     ";;
        os_coordinate << ostr_nodes.str() << endl;
        ostr_nodes.str("");
        count=count+1;
    }
    os_coordinate.close(); 
}
  
void readFlac::writeElementsFile(string elements_file_name)
{
    fstream os_elements;
    os_elements.open(elements_file_name.c_str(), ios::out);
    ostringstream ostr_elements; 
    
    if(m_cell_type == 1)
    {
        int quad[8];
        os_elements << "      " << to_string(vec_elements.size()/8) << "       " << endl;
        os_elements << endl;

        int count = 0;       
        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        {
            quad[0]=*iter++;
            quad[1]=*iter++;
            quad[2]=*iter++;
            quad[3]=*iter++;
            quad[4]=*iter++;
            quad[5]=*iter++;
            quad[6]=*iter++;
            quad[7]=*iter++;
            ostr_elements<<count + 1<<" "<<1<<" ";
            ostr_elements<<quad[0]<<" ";
            ostr_elements<<quad[1]<<" ";
            ostr_elements<<quad[3]<<" ";
            ostr_elements<<quad[2]<<" ";
            ostr_elements<<quad[3]<<" ";
            ostr_elements<<quad[6]<<" ";
            ostr_elements<<quad[7]<<" ";
            ostr_elements<<quad[5]<<" ";
       
            os_elements << ostr_elements.str() << endl;
            ostr_elements.str("");
	    count =count +1;
        }
    }
    
    if(m_cell_type == 0)
    {
        int quad[4];
        os_elements << "      " << to_string(vec_elements.size()/4) << "       " << endl;
        os_elements << endl;

        int count = 0;
        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        {
            quad[0]=*iter++;
            quad[1]=*iter++;
            quad[2]=*iter++;
            quad[3]=*iter++;
            ostr_elements<<count + 1<<" "<<1<<" ";
            ostr_elements<<quad[0]<<" ";
            ostr_elements<<quad[1]<<" ";
            ostr_elements<<quad[3]<<" ";
            ostr_elements<<quad[2]<<" ";
            os_elements << ostr_elements.str() << endl;
            ostr_elements.str("");
	    count =count +1;
        }
    }
    os_elements.close();
}

void readFlac::writeFile(string file_name)
{
    if(m_write_xml_type ==0)
    {
        writeVtkFile(file_name);
    }
    else if(m_write_xml_type ==1)
    {
        writeVtuFile(file_name);
    }
}

void readFlac::setWriteXmlMode(int write_xml_type)
{
    m_write_xml_type = write_xml_type;
}

void readFlac::setWriteDataMode(int write_data_type)
{
    m_write_data_type = write_data_type;
}

void readFlac::setWriteDataModeToAscii()
{
    m_write_data_type = 0;
}

void readFlac::setWriteDataModeToBinary()
{
    m_write_data_type = 1;
}

void readFlac::setWriteXmlModeToVtu()
{
    m_write_xml_type = 1;
}

void readFlac::setWriteXmlModeToVtk()
{
    m_write_xml_type = 0;
}

void readFlac::writeVtkFile(string vtk_file_name)
{
    vtkSmartPointer<vtkUnstructuredGridWriter> writer = vtkSmartPointer<vtkUnstructuredGridWriter>::New();
    //writer->SetFileTypeToASCII();    //1
    //writer->SetFileTypeToBinary();     //2
    writer->SetFileType(m_write_data_type+1);
    //std::cout<<writer->GetFileType()<<std::endl;
    writer->SetFileName(vtk_file_name.c_str());
    writer->SetInputData(m_grid);
    writer->Write();
}

void readFlac::writeVtuFile(string vtu_file_name)
{
    vtkSmartPointer<vtkXMLUnstructuredGridWriter> writer = vtkSmartPointer<vtkXMLUnstructuredGridWriter>::New();
    writer->SetFileName(vtu_file_name.c_str());
    //writer->SetDataType(m_write_data_type);
    writer->SetDataModeToAscii();  //0
    //writer->SetDataModeToBinary(); //1
    //std::cout<<writer->GetDataMode()<<std::endl;
    writer->SetInputData(m_grid);
    writer->Write();
}

void readFlac::setColor(float color_x, float color_y, float color_z)
{  
    m_color_x = color_x;
    m_color_y = color_y;
    m_color_z = color_z;
}

void readFlac::setOpacity(float opacity)
{  
    m_opacity =  opacity;
}

void readFlac::setLineWidth(float line_width)
{  
    m_line_width = line_width;
}

void readFlac::update()
{  
    m_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    m_actor->GetProperty()->SetOpacity(m_opacity);
}

void readFlac::updateColor(float color_x, float color_y, float color_z)
{  
    m_actor->GetProperty()->SetColor(color_x, color_y, color_z);
}

void readFlac::updateOpacity(float opacity)
{  
    m_actor->GetProperty()->SetOpacity(opacity);
}

void readFlac::updateLineWidth(float line_width)
{  
    m_actor->GetProperty()->SetLineWidth(line_width);
}

bool readFlac::endWith(const string &str, const string &tail)
{
    return str.compare(str.size() - tail.size(), tail.size(), tail) == 0;
}
 
bool readFlac::startWith(const string &str, const string &head) 
{
    return str.compare(0, head.size(), head) == 0;
}

void readFlac::split(const string &strtem, const char a, vector<string> &res)
{
    res.clear();
    string::size_type pos1, pos2;
    pos2 = strtem.find(a);
    pos1 = 0;
    while (string::npos != pos2)
    {
        res.push_back(strtem.substr(pos1, pos2 - pos1));
	pos1 = pos2 + 1;
	pos2 = strtem.find(a, pos1);
    }
    res.push_back(strtem.substr(pos1));
}

void readFlac::removeLineCarriage(std::string *line)
{
    auto p = line->end() - 1;
    while ( (*p) == '\r' || (*p) == '\n' )
    {
        line->pop_back();
        p = line->end() - 1;
    }
}









