#include<iostream>
#include<fstream>
#include<string>
#include<sstream>

#include<vtkProperty.h>
#include<vtkActor.h>
#include<vtkDataSetMapper.h>
#include<vtkPoints.h>
#include<vtkCellData.h>
#include<vtkCellArray.h>
#include<vtkQuad.h>
#include<vtkHexahedron.h>
#include<vtkUnstructuredGrid.h>
#include<vtkUnstructuredGridWriter.h>
#include<vtkXMLUnstructuredGridWriter.h>

#include "readGrid.h"
using namespace std;

readGrid::readGrid()
{
    m_actor = vtkSmartPointer<vtkActor>::New();
    m_grid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    m_color_x = 0.75;
    m_color_y = 0.75;
    m_color_z = 0.75;
    m_opacity = 1.0;
    m_line_width = 1.0;
    m_write_xml_type = 1;    //1 vtu 0 vtk
    m_write_data_type = 1;   //1 binary  0 ascii
}

readGrid::~readGrid()
{

}

bool readGrid::readGridFile(string nodepath,string elementpath)
{
    ifstream nodes(nodepath, ios::in);
    ifstream elements(elementpath, ios::in);
    if (!nodes.is_open())
    {
        return false;
    }
    
    if (!elements.is_open())
    {
        return false;
    }
    
    vtkSmartPointer<vtkPoints> vtk_points = vtkSmartPointer<vtkPoints>::New();
    vtkSmartPointer<vtkCellArray> cells = vtkSmartPointer<vtkCellArray>::New();
    //vtkSmartPointer<vtkUnstructuredGrid> poly = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    bool result1 = buildVtkPoints(nodes, vtk_points);
    if (!result1)
    {
        return false;
    }
    m_grid->SetPoints(vtk_points);
    
    bool result2 = buildVtkCells(elements, cells, m_grid);
    if (!result2)
    {
        return false;
    }
    nodes.close();
    elements.close();
	
    vtkSmartPointer<vtkDataSetMapper>mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    mapper->SetInputData(m_grid);
    //m_actor->GetProperty()->EdgeVisibilityOn();
    m_actor->SetMapper(mapper);
    m_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    m_actor->GetProperty()->SetOpacity(m_opacity);
    m_actor->GetProperty()->SetRepresentationToWireframe();
    return true;
}

vtkSmartPointer<vtkActor> readGrid::getActor()
{  
    return m_actor;
}

bool readGrid::buildVtkPoints(ifstream& nodes,  vtkSmartPointer<vtkPoints> points)
{
    string strLine;
    if (!getline(nodes, strLine))
    {
        return false;
    }
    int NPoin = stoi(strLine);
    //cout << "the number of points is" << NPoin << endl;
    //skip space line
    getline(nodes, strLine);
    double point[3];
    while (NPoin--)
    {
        getline(nodes, strLine);
	istringstream iss(strLine);
	string tem;
	//获取序号
	iss >> tem;
	//获取x,y,z值
	iss >> tem;
	point[0] = atof(tem.c_str());
	iss >> tem;
	point[1] = atof(tem.c_str());
	iss >> tem;
	point[2] = atof(tem.c_str());
	points->InsertNextPoint(point);
	//cout << point[0] << " " << point[1] << " " << point[2] << endl;	
    }
    return true;
}

bool readGrid::buildVtkCells(ifstream& elements,  vtkSmartPointer<vtkCellArray> cells,vtkSmartPointer<vtkUnstructuredGrid> Grid)
{
    string strLine;
    if (!getline(elements, strLine))
    {
        return false;
    }
    int NElem = stoi(strLine);
    //cout << "the number of element is" << NElem << endl;
    //skip space line
    getline(elements, strLine);
    
    getline(elements, strLine);
    vector<int>elementparts;
    istringstream iss(strLine);
    string tem;

    while (iss>>tem)
    {
        elementparts.push_back(stoi(tem));
    }
    //std::cout<<elementparts.size()<<std::endl;
	
    if (elementparts.size() == 6)
    {
        buildQuadcells(NElem, elements, cells, Grid);
    }
    else if(elementparts.size()==10)
    {
        buildHexcells(NElem, elements, cells, Grid);
    }
    return true;
}

void readGrid::writeFile(string file_name)
{
    if(m_write_xml_type ==0)
    {
        writeVtkFile(file_name);
    }
    else if(m_write_xml_type ==1) 
    {
        writeVtuFile(file_name);
    }
}

void readGrid::setWriteXmlMode(int write_xml_type)
{
    m_write_xml_type = write_xml_type;
}

void readGrid::setWriteDataMode(int write_data_type)
{
    m_write_data_type = write_data_type;
}

void readGrid::setWriteDataModeToAscii()
{
    m_write_data_type = 0;
}

void readGrid::setWriteDataModeToBinary()
{
    m_write_data_type = 1;
}

void readGrid::setWriteXmlModeToVtu()
{
    m_write_xml_type = 1;
}

void readGrid::setWriteXmlModeToVtk()
{
    m_write_xml_type = 0;
}

void readGrid::writeVtkFile(string vtk_file_name)
{
    vtkSmartPointer<vtkUnstructuredGridWriter> writer = vtkSmartPointer<vtkUnstructuredGridWriter>::New();
    //writer->SetFileTypeToASCII();    //1
    //writer->SetFileTypeToBinary();     //2
    writer->SetFileType(m_write_data_type+1);
    //std::cout<<writer->GetFileType()<<std::endl;
    writer->SetFileName(vtk_file_name.c_str());
    writer->SetInputData(m_grid);
    writer->Write();
}

void readGrid::writeVtuFile(string vtu_file_name)
{
    vtkSmartPointer<vtkXMLUnstructuredGridWriter> writer = vtkSmartPointer<vtkXMLUnstructuredGridWriter>::New();
    writer->SetFileName(vtu_file_name.c_str());
    //writer->SetDataType(m_write_data_type);
    writer->SetDataModeToAscii();  //0
    //writer->SetDataModeToBinary(); //1
    //std::cout<<writer->GetDataMode()<<std::endl;
    writer->SetInputData(m_grid);
    writer->Write();
}

void readGrid::buildQuadcells(int NElem, ifstream& elements,  vtkSmartPointer<vtkCellArray> cells, vtkSmartPointer<vtkUnstructuredGrid> Grid)
{
    elements.seekg(0, ios::beg);
    string strLine;
    getline(elements, strLine);
    getline(elements, strLine);

    vector<int>quad(4, 0);
    while (NElem--)
    {
        vtkSmartPointer<vtkQuad> quadpoly = vtkSmartPointer<vtkQuad>::New();
	getline(elements, strLine);
	istringstream iss(strLine);
	string tem;

	iss >> tem;
	iss >> tem;
	for (int i = 0;i < 4;i++)
	{
	    iss >> tem;
	    quad[i] = stoi(tem) - 1;
	    quadpoly->GetPointIds()->SetId(i, quad[i]);
	}
			
	//cout << quad[0] << " " << quad[1] << " " << quad[2] << " " << quad[3] << endl;
	cells->InsertNextCell(quadpoly);
	Grid->InsertNextCell(quadpoly->GetCellType(), quadpoly->GetPointIds());
	//cout << "quadcelltypenumber is " << quadpoly->GetCellType() << endl;
			
    }
}

void readGrid::buildHexcells(int NElem, ifstream& elements, vtkSmartPointer<vtkCellArray> cells, vtkSmartPointer<vtkUnstructuredGrid> Grid)
{
    elements.seekg(0, ios::beg);
    string strLine;
    getline(elements, strLine);
    getline(elements, strLine);

    vector<int>hexah(8, 0);
    while (NElem--)
    {
        vtkSmartPointer<vtkHexahedron> hexpoly = vtkSmartPointer<vtkHexahedron>::New();
        getline(elements, strLine);
	istringstream iss(strLine);
	string tem;
	
	iss >> tem;
	iss >> tem;
	for (int i = 0;i < 8;i++)
	{
            iss >> tem;
	    hexah[i] = stoi(tem) - 1;
	    hexpoly->GetPointIds()->SetId(i, hexah[i]);
	    //cout << hexah[0] << " " << hexah[1] << " " << hexah[2] << " " << hexah[3] << " " << hexah[4] << " " << hexah[5] << " " << hexah[6] << " " << hexah[7] << endl;
	}
	cells->InsertNextCell(hexpoly);
	Grid->InsertNextCell(hexpoly->GetCellType(), hexpoly->GetPointIds());
	//cout << "hexpolycelltypenumber is " << hexpoly->GetCellType() << endl;
    }
}

void readGrid::setColor(float color_x, float color_y, float color_z)
{
    m_color_x = color_x;
    m_color_y = color_y;
    m_color_z = color_z;
}

void readGrid::setOpacity(float opacity)
{
    m_opacity =  opacity;
}

void readGrid::setLineWidth(float line_width)
{
    m_line_width = line_width;
}

void readGrid::update()
{
    m_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    m_actor->GetProperty()->SetOpacity(m_opacity);
}

void readGrid::updateColor(float color_x, float color_y, float color_z)
{
    m_actor->GetProperty()->SetColor(color_x, color_y, color_z);
}

void readGrid::updateOpacity(float opacity)
{
    m_actor->GetProperty()->SetOpacity(opacity);
}

void readGrid::updateLineWidth(float line_width)
{
    m_actor->GetProperty()->SetLineWidth(line_width);
}





