#include<iostream>
#include<fstream>
#include<string>
#include<sstream>
#include<vector> 

#include<vtkProperty.h>
#include<vtkActor.h>
#include<vtkDataSetMapper.h>
#include<vtkPoints.h>
#include<vtkCellData.h>
#include<vtkCellArray.h>
#include<vtkQuad.h>
#include<vtkHexahedron.h>
#include<vtkUnstructuredGrid.h>
#include<vtkUnstructuredGridWriter.h>
#include<vtkXMLUnstructuredGridWriter.h>
#include <qfile.h>
#include <QTextStream>
#include <vtkTetra.h>
#include <vtkTriangle.h>
#include <vtkTriangle.h>
#include <vtkStructuredGrid.h>
#include <vtkLine.h>
//#include <vtkUnStructuredGrid.h>

#include "qdebug.h"
#include "readNas.h"
using namespace std;

readNas::readNas()
{
    m_actor = vtkSmartPointer<vtkActor>::New();
    m_grid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    
    m_color_x = 0.75;
    m_color_y = 0.75;
    m_color_z = 0.75;
    m_opacity = 1.0;
    m_line_width = 1.0;
    m_write_xml_type = 1;    //1 vtu 0 vtk
    m_write_data_type = 1;   //1 binary  0 ascii
    
    m_cell_type = -1;
    
    vec_points.clear();
    vec_elements.clear();
}

readNas::~readNas()
{

}

bool readNas::readNasFile(string nas_file_name)
{
    QFile file(QString::fromStdString(nas_file_name));
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {

        return false;
    }
    int count1 = 0;
    int count2 = 0;

    QTextStream in(&file);
    while (!in.atEnd()) {
        QString line = in.readLine().trimmed();
        if (line.isEmpty())
            continue;

        QStringList tokens = line.split(QRegExp("\\s+"), QString::SkipEmptyParts);
        QString keyword = tokens[0];

        if (keyword == "GRID") {

            QString x_str = line.mid(24, 8).trimmed();
            QString y_str = line.mid(32, 8).trimmed();
            QString z_str = line.mid(40, 8).trimmed();

            vec_points.push_back(x_str.toDouble());
            vec_points.push_back(y_str.toDouble());
            vec_points.push_back(z_str.toDouble());
        }
        else if (keyword == "CTETRA") {
            m_cell_type = 0;
            vec_elements.push_back(tokens[3].toInt());
            vec_elements.push_back(tokens[4].toInt());
            vec_elements.push_back(tokens[5].toInt());
            vec_elements.push_back(tokens[6].toInt());

        }
        else if (keyword == "CHEXA") {
            m_cell_type = 1;
            vec_elements.push_back(tokens[3].toInt());
            vec_elements.push_back(tokens[4].toInt());
            vec_elements.push_back(tokens[5].toInt());
            vec_elements.push_back(tokens[6].toInt());
            vec_elements.push_back(tokens[7].toInt());


            QString token8 = tokens[8];
            QString numberPart = token8.split("+CONT").first().trimmed();
            vec_elements.push_back(numberPart.toInt());

            // Read the next line and parse it
            QString CHEXALine = in.readLine().trimmed();
            QStringList CHEXATokens = CHEXALine.split(QRegExp("\\s+"), QString::SkipEmptyParts);

            if (CHEXATokens.size() >= 3) {
                vec_elements.push_back(CHEXATokens[1].toInt());
                vec_elements.push_back(CHEXATokens[2].toInt());
            } else {
                qDebug()<<"error";
            }
        }
        else if (keyword == "CQUAD4") {
            Quad quad;
            quad.mId = tokens[1].toInt();
            quad.faceId = tokens[2].toInt();
            quad.n1 = tokens[3].toInt();
            quad.n2 = tokens[4].toInt();
            quad.n3 = tokens[5].toInt();
            quad.n4 = tokens[6].toInt();
            quads.push_back(quad);
        }
        else if (keyword == "CTRIA3") {
            Triangle tri;
            tri.mId = tokens[1].toInt();
            tri.faceId = tokens[2].toInt();
            tri.n1 = tokens[3].toInt();
            tri.n2 = tokens[4].toInt();
            tri.n3 = tokens[5].toInt();
            triangles.push_back(tri);
        }
        else if(keyword == "PSHELL")
        {
            m_faceNum = tokens[1].toInt();
        }
    }
    file.close();


    //    for(int i=0;i<test2.size();i++)
    //    {
    //        if(test[test2[i]] == 1)
    //            qDebug()<<"no use Point:"<<test2[i];
    //    }

    m_faceToPoint.resize(m_faceNum+1);
    m_faceToCell.resize(m_faceNum+1);






    if(m_cell_type == 0) //
    {
        m_gridHasPoint.resize(vec_elements.size()/4+1);
        int quad[4];
        int index = 1;
        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        {
            vtkSmartPointer<vtkHexahedron> hexpoly = vtkSmartPointer<vtkHexahedron>::New();
            quad[0]=*iter++;
            quad[1]=*iter++;
            quad[2]=*iter++;
            quad[3]=*iter++;

            m_gridHasPoint[index][quad[0]] = true;
            m_gridHasPoint[index][quad[1]] = true;
            m_gridHasPoint[index][quad[2]] = true;
            m_gridHasPoint[index][quad[3]] = true;
            index++;
        }
    }

    else
    {
        m_gridHasPoint.resize(vec_elements.size()/8+1);
        int quad[8];
        int index = 1;
        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        {
            quad[0]=*iter++;
            quad[1]=*iter++;
            quad[2]=*iter++;
            quad[3]=*iter++;
            quad[4]=*iter++;
            quad[5]=*iter++;
            quad[6]=*iter++;
            quad[7]=*iter++;

            m_gridHasPoint[index][quad[0]] = true;
            m_gridHasPoint[index][quad[1]] = true;
            m_gridHasPoint[index][quad[2]] = true;
            m_gridHasPoint[index][quad[3]] = true;
            m_gridHasPoint[index][quad[4]] = true;
            m_gridHasPoint[index][quad[5]] = true;
            m_gridHasPoint[index][quad[6]] = true;
            m_gridHasPoint[index][quad[7]] = true;
            index++;
        }
    }
    processVtkStruct();
    return true;






    //    vec_points.clear();
    //    vec_elements.clear();
    //    ifstream ifs;
    //    ifs.open(nas_file_name.c_str(), std::ios::in);
    //    if(!ifs.is_open() )
    //    {
    //        cerr << "Error: Cannot open the file '" << nas_file_name << "'." << endl;
    //        return false;
    //    }
    //    string line;
    //    int count1 = 0;
    //    int count2 = 0;
    //    while(std::getline(ifs, line))
    //    {
    //        removeLineCarriage(&line);
    //        if(startWith(line, string("GRID")))
    //        {
    //            //line_id = line.substr(8, 8);
    //            vec_points.push_back(atof(line.substr(24, 8).c_str()));
    //            vec_points.push_back(atof(line.substr(32, 8).c_str()));
    //            vec_points.push_back(atof(line.substr(40, 8).c_str()));
    //        }

    //        if(startWith(line, string("GRID*")))
    //        {
    //            //line_id = line.substr(8, 16);
    //            vec_points.push_back(atof(line.substr(40, 16).c_str()));
    //            vec_points.push_back(atof(line.substr(56, 16).c_str()));
    //            std::getline(ifs, line);
    //            vec_points.push_back(atof(line.substr(8, 16).c_str()));
    //        }

    //        if(startWith(line, string("CQUAD4")))
    //        {
    //            count1++;
    //            vec_elements.push_back(atoi(line.substr(24, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(32, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(40, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(48, 8).c_str()));
    //        }

    //        if(startWith(line, string("CHEXA")))
    //        {
    //            count2++;
    //            vec_elements.push_back(atoi(line.substr(24, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(32, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(40, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(48, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(56, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(64, 8).c_str()));
    //            std::getline(ifs, line);
    //            vec_elements.push_back(atoi(line.substr(8, 8).c_str()));
    //            vec_elements.push_back(atoi(line.substr(16, 8).c_str()));
    //            //std::cout<<line.substr(16, 8)<<std::endl;
    //        }
    //    }
    //    if(count1>0)
    //    {
    //        m_cell_type = 0;
    //    }
    
    //    if(count2>0)
    //    {
    //        m_cell_type = 1;
    //    }
    //    ifs.close();

    //    return true;
}

void readNas::processVtkStruct()
{
    vtkSmartPointer<vtkPoints> points = vtkSmartPointer<vtkPoints>::New();
    vtkSmartPointer<vtkCellArray> cells = vtkSmartPointer<vtkCellArray>::New();

    //--------------test;
    test = vtkSmartPointer<vtkUnstructuredGrid>::New();


    float point[3];
    for (auto iter = vec_points.begin(); iter != vec_points.end();)
    {
        point[0]=*iter++;
        point[1]=*iter++;
        point[2]=*iter++;
        points->InsertNextPoint(point);
    }

    m_grid->SetPoints(points);
    //--------------test;
    test->SetPoints(points);
    //qDebug()<<QStringLiteral("idid");
    if(m_cell_type ==1)//
    {
        for(int j=0;j<quads.size();j++)//͵
        {
            vtkSmartPointer<vtkQuad> quadpoly = vtkSmartPointer<vtkQuad>::New();
            Quad tmp =  quads[j];

            quadpoly->GetPointIds()->SetId(0, tmp.n1-1);
            quadpoly->GetPointIds()->SetId(1, tmp.n2-1);
            quadpoly->GetPointIds()->SetId(2, tmp.n3-1);
            quadpoly->GetPointIds()->SetId(3, tmp.n4-1);


            int id = m_grid->InsertNextCell(quadpoly->GetCellType(), quadpoly->GetPointIds());
            m_faceToCell[tmp.faceId].push_back(id+1);

            //Ѱڼеid
            m_cellIdToGridCellId[id] = -1;//


            for(int i=1;i<m_gridHasPoint.size();i++)
            {
                if(m_gridHasPoint[i][tmp.n1] && m_gridHasPoint[i][tmp.n2] &&
                        m_gridHasPoint[i][tmp.n3] &&m_gridHasPoint[i][tmp.n4])//
                {
                    m_cellIdToGridCellId[id] = i;
                    break;
                }
            }

            m_faceToPoint[tmp.faceId].insert(tmp.n1);
            m_faceToPoint[tmp.faceId].insert(tmp.n2);
            m_faceToPoint[tmp.faceId].insert(tmp.n3);
            m_faceToPoint[tmp.faceId].insert(tmp.n4);
        }

    }

    if(m_cell_type ==0)//
    {
        int quad[4];

        for(int j=0;j<triangles.size();j++)//͵
        {
            vtkSmartPointer<vtkTriangle> quadpoly = vtkSmartPointer<vtkTriangle>::New();
            Triangle tmp =  triangles[j];

            quadpoly->GetPointIds()->SetId(0, tmp.n1-1);
            quadpoly->GetPointIds()->SetId(1, tmp.n2-1);
            quadpoly->GetPointIds()->SetId(2, tmp.n3-1);

            int id = m_grid->InsertNextCell(quadpoly->GetCellType(), quadpoly->GetPointIds());


            //--------------test;
            vtkSmartPointer<vtkLine> tt = vtkSmartPointer<vtkLine>::New();
            tt->GetPointIds()->SetId(0, tmp.n1-1);
            tt->GetPointIds()->SetId(1, tmp.n2-1);
            test->InsertNextCell(tt->GetCellType(),tt->GetPointIds());

            vtkSmartPointer<vtkLine> tt2 = vtkSmartPointer<vtkLine>::New();
            tt2->GetPointIds()->SetId(0, tmp.n2-1);
            tt2->GetPointIds()->SetId(1, tmp.n3-1);
            test->InsertNextCell(tt2->GetCellType(),tt2->GetPointIds());

            vtkSmartPointer<vtkLine> tt3 = vtkSmartPointer<vtkLine>::New();
            tt3->GetPointIds()->SetId(0, tmp.n3-1);
            tt3->GetPointIds()->SetId(1, tmp.n1-1);
            test->InsertNextCell(tt3->GetCellType(),tt3->GetPointIds());

            //---------------------Ѱڼеid
            m_cellIdToGridCellId[id] = -1;//

            for(int i=1;i<m_gridHasPoint.size();i++)
            {
                if(m_gridHasPoint[i][tmp.n1] && m_gridHasPoint[i][tmp.n2] &&
                        m_gridHasPoint[i][tmp.n3] )//
                {
                    m_cellIdToGridCellId[id] = i;
                    break;
                }
            }
            //---------------------Ѱڼеid
            m_faceToCell[tmp.faceId].push_back(id+1);

            m_faceToPoint[tmp.faceId].insert(tmp.n1);
            m_faceToPoint[tmp.faceId].insert(tmp.n2);
            m_faceToPoint[tmp.faceId].insert(tmp.n3);
        }
        //qDebug()<<QStringLiteral("idid");




        //        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        //        {
        //            vtkSmartPointer<vtkTetra> quadpoly = vtkSmartPointer<vtkTetra>::New();
        //            quad[0]=*iter++;
        //            quad[1]=*iter++;
        //            quad[2]=*iter++;
        //            quad[3]=*iter++;
        //            quadpoly->GetPointIds()->SetId(0, quad[0]-1);
        //            quadpoly->GetPointIds()->SetId(1, quad[1]-1);
        //            quadpoly->GetPointIds()->SetId(2, quad[2]-1);
        //            quadpoly->GetPointIds()->SetId(3, quad[3]-1);
        //            m_grid->InsertNextCell(quadpoly->GetCellType(), quadpoly->GetPointIds());
        //        }
    }
    //    vtkSmartPointer<vtkDataSetMapper>mapper = vtkSmartPointer<vtkDataSetMapper>::New();
    //    mapper->SetInputData(m_grid);
    //    //m_actor->GetProperty()->EdgeVisibilityOn();
    //    m_actor->SetMapper(mapper);
    //    m_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    //    m_actor->GetProperty()->SetOpacity(m_opacity);
    //    m_actor->GetProperty()->SetRepresentationToWireframe();
}

QMap<int,int> readNas::getCellIdToGridIdMap()
{
    return m_cellIdToGridCellId;
}

vtkSmartPointer<vtkActor> readNas::getActor()
{  
    return m_actor;
}

vtkSmartPointer<vtkUnstructuredGrid> readNas::getTest()
{
    return test;
}

vtkSmartPointer<vtkUnstructuredGrid> readNas::getStructure()
{
    return m_grid;
}

QVector<QSet<int>> readNas::getFaceToPoint()
{
    return m_faceToPoint;
}

QVector<QVector<int>> readNas::getFaceToCell()
{
    return m_faceToCell;
}

void readNas::writeGridFile(string nodes_file_name,string elements_file_name)
{
    writeNodesFile(nodes_file_name);
    writeElementsFile(elements_file_name);
}

void readNas::writeNodesFile(string nodes_file_name)
{
    fstream os_coordinate;
    os_coordinate.open(nodes_file_name.c_str(), ios::out);
    ostringstream ostr_nodes;
    float point[3];
    int count = 0;
    for (auto iter = vec_points.begin(); iter != vec_points.end();)
    {
        point[0]=*iter++;
        point[1]=*iter++;
        point[2]=*iter++;
        ostr_nodes<<"      ";
        ostr_nodes<<count + 1<<"     ";
        ostr_nodes<<setiosflags(ios::fixed)<<setiosflags(ios::right)<<setprecision(8)<<point[0]<<"     ";
        ostr_nodes<<setiosflags(ios::fixed)<<setiosflags(ios::right)<<setprecision(8)<<point[1]<<"     ";;
        ostr_nodes<<setiosflags(ios::fixed)<<setiosflags(ios::right)<<setprecision(8)<<point[2]<<"     ";;
        ostr_nodes.str("");
        count=count+1;
    }
    os_coordinate.close();
}

void readNas::writeElementsFile(string elements_file_name)
{
    fstream os_elements;
    os_elements.open(elements_file_name.c_str(), ios::out);
    ostringstream ostr_elements;
    
    if(m_cell_type == 1)
    {
        int quad[8];

        int count = 0;
        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        {
            quad[0]=*iter++;
            quad[1]=*iter++;
            quad[2]=*iter++;
            quad[3]=*iter++;
            quad[4]=*iter++;
            quad[5]=*iter++;
            quad[6]=*iter++;
            quad[7]=*iter++;
            ostr_elements<<count + 1<<" "<<1<<" ";
            ostr_elements<<quad[0]<<" ";
            ostr_elements<<quad[1]<<" ";
            ostr_elements<<quad[2]<<" ";
            ostr_elements<<quad[3]<<" ";
            ostr_elements<<quad[4]<<" ";
            ostr_elements<<quad[5]<<" ";
            ostr_elements<<quad[6]<<" ";
            ostr_elements<<quad[7]<<" ";

            ostr_elements.str("");
            count =count +1;
        }
    }
    
    if(m_cell_type == 0)
    {
        int quad[4];
        int count = 0;
        for (auto iter = vec_elements.begin(); iter != vec_elements.end();)
        {
            quad[0]=*iter++;
            quad[1]=*iter++;
            quad[2]=*iter++;
            quad[3]=*iter++;
            ostr_elements<<count + 1<<" "<<1<<" ";
            ostr_elements<<quad[0]<<" ";
            ostr_elements<<quad[1]<<" ";
            ostr_elements<<quad[2]<<" ";
            ostr_elements<<quad[3]<<" ";
            ostr_elements.str("");
            count =count +1;
        }
    }
    os_elements.close();
}

void readNas::writeFile(string file_name)
{
    if(m_write_xml_type ==0)
    {
        writeVtkFile(file_name);
    }
    else if(m_write_xml_type ==1)
    {
        writeVtuFile(file_name);
    }
}

void readNas::setWriteXmlMode(int write_xml_type)
{
    m_write_xml_type = write_xml_type;
}

void readNas::setWriteDataMode(int write_data_type)
{
    m_write_data_type = write_data_type;
}

void readNas::setWriteDataModeToAscii()
{
    m_write_data_type = 0;
}

void readNas::setWriteDataModeToBinary()
{
    m_write_data_type = 1;
}

void readNas::setWriteXmlModeToVtu()
{
    m_write_xml_type = 1;
}

void readNas::setWriteXmlModeToVtk()
{
    m_write_xml_type = 0;
}

void readNas::writeVtkFile(string vtk_file_name)
{
    vtkSmartPointer<vtkUnstructuredGridWriter> writer = vtkSmartPointer<vtkUnstructuredGridWriter>::New();
    //writer->SetFileTypeToASCII();    //1
    //writer->SetFileTypeToBinary();     //2
    writer->SetFileType(m_write_data_type+1);
    //std::cout<<writer->GetFileType()<<std::endl;
    writer->SetFileName(vtk_file_name.c_str());
    writer->SetInputData(m_grid);
    writer->Write();
}

void readNas::writeVtuFile(string vtu_file_name)
{
    vtkSmartPointer<vtkXMLUnstructuredGridWriter> writer = vtkSmartPointer<vtkXMLUnstructuredGridWriter>::New();
    writer->SetFileName(vtu_file_name.c_str());
    //writer->SetDataType(m_write_data_type);
    writer->SetDataModeToAscii();  //0
    //writer->SetDataModeToBinary(); //1
    //std::cout<<writer->GetDataMode()<<std::endl;
    writer->SetInputData(m_grid);
    writer->Write();
}

void readNas::setColor(float color_x, float color_y, float color_z)
{  
    m_color_x = color_x;
    m_color_y = color_y;
    m_color_z = color_z;
}

void readNas::setOpacity(float opacity)
{  
    m_opacity =  opacity;
}

void readNas::setLineWidth(float line_width)
{  
    m_line_width = line_width;
}

void readNas::update()
{  
    m_actor->GetProperty()->SetColor(m_color_x, m_color_y, m_color_z);
    m_actor->GetProperty()->SetOpacity(m_opacity);
}

void readNas::updateColor(float color_x, float color_y, float color_z)
{  
    m_actor->GetProperty()->SetColor(color_x, color_y, color_z);
}

void readNas::updateOpacity(float opacity)
{  
    m_actor->GetProperty()->SetOpacity(opacity);
}

void readNas::updateLineWidth(float line_width)
{  
    m_actor->GetProperty()->SetLineWidth(line_width);
}

bool readNas::endWith(const string &str, const string &tail)
{
    return str.compare(str.size() - tail.size(), tail.size(), tail) == 0;
}

bool readNas::startWith(const string &str, const string &head) 
{
    return str.compare(0, head.size(), head) == 0;
}

void readNas::split(const string &strtem, const char a, vector<string> &res)
{
    res.clear();
    string::size_type pos1, pos2;
    pos2 = strtem.find(a);
    pos1 = 0;
    while (string::npos != pos2)
    {
        res.push_back(strtem.substr(pos1, pos2 - pos1));
        pos1 = pos2 + 1;
        pos2 = strtem.find(a, pos1);
    }
    res.push_back(strtem.substr(pos1));
}

void readNas::removeLineCarriage(std::string *line)
{
    auto p = line->end() - 1;
    while ( (*p) == '\r' || (*p) == '\n' )
    {
        line->pop_back();
        p = line->end() - 1;
    }
}









