#pragma once

#include<iostream>
#include<fstream>
#include<string>
#include<vector>

#include <QMap>
#include <QVector>
#include<vtkActor.h>
#include<vtkPoints.h>
#include<vtkUnstructuredGrid.h>

using namespace std;

struct Quad
{
    int mId,faceId;
    int n1,n2,n3,n4;
};
struct Triangle
{
    int mId,faceId;
    int n1,n2,n3;
};

class readNas
{
public:
    readNas();
    ~readNas();
    
private:
    vtkSmartPointer<vtkActor> m_actor;
    vtkSmartPointer<vtkUnstructuredGrid> m_grid;
    
    float m_color_x;
    float m_color_y;
    float m_color_z;
    float m_opacity;
    float m_line_width;
    
    //1 vtu 0 vtk
    int m_write_xml_type;
    //1 binary  0 ascii
    int m_write_data_type;
    
    //0 quas 1 hex
    int m_cell_type; 

    //
    int m_faceNum;
    
    vector<float> vec_points;
    vector<int> vec_elements;
    vector<Quad> quads;
    vector<Triangle> triangles;
    QVector<QVector<int>> m_faceToCell;
    QMap<int,int>m_cellIdToGridCellId;
    QMap<int,int>m_PointIdToGridPointId;
    QVector<QSet<int>> m_faceToPoint;

    QVector<QMap<int,bool>>m_gridHasPoint;

    
    bool endWith(const string &str, const string &tail);
    bool startWith(const string &str, const string &tail);
    void split(const string &strtem, const char a, vector<string> &res);
    void removeLineCarriage(std::string *line);
    
public:
    vtkSmartPointer<vtkActor> getActor();
    bool readNasFile(string nas_file_name);
    
    void setColor(float color_x, float color_y, float color_z);
    void setOpacity(float opacity);    
    void setLineWidth(float line_width);
    
    float getOpacity()const{return m_opacity;}
    float getLineWidth()const{return m_line_width;}
    
    int getWriteDataMode()const{return m_write_data_type;}
    int getWriteXmlMode()const{return m_write_xml_type;}

    int getCellType()const{return m_cell_type;}
    
    void setWriteXmlMode(int write_xml_type);
    void setWriteDataMode(int write_data_type);
    //setWriteDataMode(0)
    void setWriteDataModeToAscii();
    //setWriteDataMode(1)
    void setWriteDataModeToBinary();
    //setWriteXmlMode(1)
    void setWriteXmlModeToVtu();
    //setWriteXmlMode(0)
    void setWriteXmlModeToVtk();
    
    void update();
    void updateColor(float color_x, float color_y, float color_z);
    void updateOpacity(float opacity);
    void updateLineWidth(float line_width);
    
    //write vtu file,if the xml_mode is 1
    //write vtk file,if the xml_mode is 0
    void writeFile(string file_name);
    //write vtu file,but not change the xml_mode
    void writeVtuFile(string vtu_file_name);
    //write vtk file,but not change the xml_mode
    void writeVtkFile(string vtk_file_name);
    
    void processVtkStruct();

    void writeGridFile(string nodes_file_name,string elements_file_name);
    void writeNodesFile(string nodes_file_name);
    void writeElementsFile(string elements_file_name);
    vtkSmartPointer<vtkUnstructuredGrid> getStructure();
    QVector<QSet<int> > getFaceToPoint();
    QVector<QVector<int> > getFaceToCell();
    QMap<int, int> getCellIdToGridIdMap();
    vtkSmartPointer<vtkUnstructuredGrid> test;
    vtkSmartPointer<vtkUnstructuredGrid> getTest();
};
















