﻿#include "ProjectManager.h"
#include "ProjectManagerWidget.h"
#include <QVBoxLayout>
#include <qheaderview.h>
#include <public/services/UiFrameService.h>
#include <utils/ServiceRegistry.h>
#include <QAction>
#include <QDebug>
#include <QMenu>
#include <QToolButton>
#include <QLabel>
#include <plugins/CA_GlobalDefinition/FunctionWidget.h>
#include <plugins/CA_PreProcessing/BuildGridSettingWidget.h>
#include <plugins/CA_PreProcessing/CuboidSettingWidget.h>
#include <plugins/CA_PreProcessing/ImportGridSettingWidget.h>
#include <plugins/CA_PreProcessing/PreProcessing.h>
#include <plugins/CA_PreProcessing/RectangleSettingWidget.h>
#include <plugins/CA_MaterialSetting/ConstitutiveModelSettingWidget.h>
#include <plugins/CA_MaterialSetting/HeterogeneousSettingWidget.h>
#include <plugins/CA_MaterialSetting/PhysicalParametersSettingWidget.h>
#include <plugins/CA_MaterialSetting/StrengthSettingWidget.h>
#include <plugins/CA_PhysicalField/AnalysisTypeSettingWidget.h>
#include <plugins/CA_PhysicalField/BoundarySettingWidget.h>
#include <plugins/CA_PhysicalField/ExcavateSettingWidget.h>
#include <plugins/CA_PhysicalField/StaticProblemsSettingWidget.h>
#include <plugins/CA_Slover/CalculateSettingWidget.h>
#include <plugins/CA_Slover/OutputSettingWidget.h>
#include <plugins/CA_PostProcessing/BisectSettingWidget.h>
#include <plugins/CA_PostProcessing/ExportSettingWidget.h>
#include <plugins/CA_PostProcessing/GraphSettingWidget.h>
#include <plugins/CA_PostProcessing/NephogramSettingWidget.h>
#include <plugins/CA_PostProcessing/SectionSettingWidget.h>
ProjectManagerWidget::ProjectManagerWidget(ProjectManager *projectManager):m_projectManager(projectManager)
{
    m_uiFrameService = ServiceRegistry<UiFrameService>::instance().getService();
    setupMenuButton();
    setupTreeWidget();
    setupSettingWidget();
    setupConnections();
    tmpInsertWidget();
}

void ProjectManagerWidget::setupMenuButton()
{
    QPushButton *button = new QPushButton(QStringLiteral("文件"));

    QString buttonstyle = "QPushButton {"
                   "    background-color: rgb(54, 140, 203); /* 设置背景色 */"
                    "  text-align: center;"
                    "padding:0px;"
                    "padding-left:5px;"
                    "font-size:17px;"
                    "font-family: Microsoft YaHei, 微软雅黑;"
                   "    color: white; /* 设置文本颜色 */"
                   "}"
                    "QPushButton::menu-indicator{image:none}"
                   "QPushButton:hover {"
                   "    background-color: #0078D7; /* 设置鼠标悬停时的背景色 */"
                   "    color: black; /* 设置鼠标悬停时的文本颜色 */"
                   "}"
                   "QPushButton:pressed {"
                   "    background-color: #005499; /* 设置按钮按下时的背景色 */"
                   "}";

    button->setStyleSheet(buttonstyle);

    QMenu *menu = new QMenu(button);

    QString style = "QMenu {"
                   "    background-color: #F0F0F0; /* 设置背景色 */"
                   "    border: 1px solid #707070; /* 设置边框 */"
                   "}"

                   "QMenu::item {"
                   "    padding: 5px 20px; /* 设置每个菜单项的内边距 */"
                   "}"
                   "QMenu::item:selected {"
                   "    background-color: #0078D7; /* 设置选中时的背景色 */"
                   "    color: white; /* 设置选中时的文本颜色 */"
                   "}";

    menu->setStyleSheet(style);

    // 创建 QActions，并添加到 QMenu
    m_newProjectAction = new QAction(QStringLiteral("新建"), menu);
    m_openProjectAction = new QAction(QStringLiteral("打开"), menu);
    m_saveProjectAction = new QAction(QStringLiteral("保存"), menu);
    m_reSaveProjectAction = new QAction(QStringLiteral("另存为"), menu);
    m_helpAction = new QAction(QStringLiteral("帮助"), menu);
    menu->addAction(m_newProjectAction);
    menu->addAction(m_openProjectAction);
    menu->addAction(m_saveProjectAction);
    menu->addAction(m_reSaveProjectAction);
    menu->addAction(m_helpAction);

    button->setMenu(menu);
    button->setFixedWidth(65);
    m_uiFrameService->addFileButton(button);
}

void ProjectManagerWidget::setupTreeWidget()
{
    //树形列表窗口
    QWidget *treeWidget = new QWidget;
    m_treeWidget = new QTreeWidget();
    m_treeWidget->setIndentation(20);
    m_treeWidget->setStyleSheet(QString(
                                    "QTreeWidget {border:none;font-family: Microsoft YaHei, 微软雅黑;}"
                                    "QTreeView{"
                                    "font-size:%2px;"
                                    "show-decoration-selected:1;"
                                    "selection-background-color:qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1,stop:0 rgba(255, 255, 255, 255), stop:0.5 rgba(235, 247, 253, 255), stop:1 rgba(213, 237, 250, 255));"
                                    "color:black; "
                                    "background-color: rgb(255, 255, 255);	"
                                    "}"
                                    "  QTreeView::item {"
                                    "      height:%1px;"
                                    "       border: 1px;"
                                    "       color:black;"
                                    "      border: transparent;"
                                    "  }"
                                    "QTreeView::item:selected,"
                                    "QTreeView::branch:selected {"
                                    "    background: rgb(192,226,252);color: black;"
                                    "}"
                                    "QTreeView::item:hover,"
                                    "QTreeView::branch:hover {"
                                    "    background: rgb(226,242,254);color: black;"
                                    "}"

                                    "QTreeView::branch {"
                                    "    color: black; /* 箭头颜色设为黑色 */"
                                    "}"
                                    "QTreeView::branch:closed:has-children {"
                                    " image: url(:/resources/btn_up.png);"
                                    "}"

                                    "QTreeView::branch:open:has-children {"
                                    "image: url(:/resources/btn_down.png);"
                                    " }"
                                    ).arg(30).arg(16));

    QHBoxLayout *mylayout = new QHBoxLayout(treeWidget);
    mylayout->addWidget(m_treeWidget);
    mylayout->setContentsMargins(3, 3, 3, 3);

    m_treeWidget->setColumnCount(1);
    m_treeWidget->header()->setVisible(false);
    m_treeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
    m_treeWidget->setSelectionMode(QAbstractItemView::SingleSelection);
    m_treeWidget->setExpandsOnDoubleClick(true);
    m_treeWidget->setEditTriggers(QAbstractItemView::EditKeyPressed);
    m_treeWidget->setHeaderLabel("Setting");
    m_rootItem = new QTreeWidgetItem();
    m_rootItem->setText(0, QStringLiteral("项目1"));

    // 添加树节点到树形列表
    m_treeWidget->addTopLevelItem(m_rootItem);

    m_uiFrameService->addWidget(treeWidget,QStringLiteral("模型开发器"));

    QList<QString>tmp;
}

void ProjectManagerWidget::generateItem(QList<QString>text,QTreeWidgetItem *parentItem)
{
    for(QString name:text)
    {
        QTreeWidgetItem *childItem = new QTreeWidgetItem(parentItem);
        childItem->setText(0, name);
    }
}

void ProjectManagerWidget::setupConnections()
{
    connect(m_treeWidget, &QTreeWidget::itemClicked, this, &ProjectManagerWidget::onItemPressed);
    connect(m_treeWidget, &QTreeWidget::customContextMenuRequested, this, &ProjectManagerWidget::generateMenu);
    connect(m_newProjectAction, &QAction::triggered, m_projectManager, &ProjectManager::newProject);
    connect(m_openProjectAction, &QAction::triggered, m_projectManager, &ProjectManager::openProject);
    connect(m_saveProjectAction, &QAction::triggered, m_projectManager, &ProjectManager::saveProject);
    connect(m_reSaveProjectAction, &QAction::triggered, m_projectManager, &ProjectManager::reSaveProject);
}

void ProjectManagerWidget::generateMenu(const QPoint &pos)
{
    QTreeWidgetItem *item = m_treeWidget->itemAt(pos);
    if (item) {
        qDebug()<<item->text(0);
    }
}

void ProjectManagerWidget::onItemPressed(QTreeWidgetItem *item, int column)
{

    QString uuid;
    QVariant retrievedData = item->data(0, Qt::UserRole);
    if (retrievedData.isValid()) {
        uuid = retrievedData.toString();
    } else {
        return;
    }

    qDebug()<<uuid;
    if(m_itemMap[uuid])
    {
        m_tmpSettingWidget->setParent(nullptr);
        m_settingLayout->removeWidget(m_tmpSettingWidget);
        m_settingWidget = m_itemMap[uuid];
        m_tmpSettingWidget = m_itemMap[uuid];
        m_settingLayout->addWidget(m_tmpSettingWidget);
        m_tmpSettingWidget->update();
    }
}

void ProjectManagerWidget::setupSettingWidget()
{
    m_settingWidget = new QWidget;

    m_settingLayout = new QVBoxLayout(m_settingWidget);
    m_tmpSettingWidget = new QWidget();
    m_settingWidget->setStyleSheet("background: rgb(255, 255, 255);");
    m_settingLayout->addWidget(m_tmpSettingWidget);
    m_settingLayout->setSpacing(0);
    m_settingLayout->setContentsMargins(0, 0, 0, 0);
    m_uiFrameService->addWidget(m_settingWidget,QStringLiteral("设置"));


}

void ProjectManagerWidget::addTreeItem(QTreeWidgetItem *treeItem)
{
    m_rootItem->addChild(treeItem);
}

void ProjectManagerWidget::tmpInsertWidget()
{

}

void ProjectManagerWidget::setItemWidget(QString uuid, QWidget *widget)
{
    m_itemMap[uuid] = widget;
}

void ProjectManagerWidget::setCurrentItem(QTreeWidgetItem *item)
{
    m_treeWidget->setCurrentItem(item);
    emit m_treeWidget->itemClicked(item, 0);
}

void ProjectManagerWidget::expandTreeWidget()
{
    m_treeWidget->expandAll();
}

