﻿#include "SystemButtonBar.h"

#include <QPushButton>
#include <qboxlayout.h>

SystemButtonBar::SystemButtonBar()
{
    m_mainLayout = new QHBoxLayout;
    m_mainLayout->setContentsMargins(0, 0, 0, 0);
    m_mainLayout->setSpacing(0);

    m_buttonClose = new QPushButton;
    m_buttonMaximize = new QPushButton;
    m_buttonMinimize = new QPushButton;
    m_buttonRestore = new QPushButton;

    QString themeStr = getTheme();

    m_buttonClose->setIcon(QIcon(":resources/close.png"));
    m_buttonMaximize->setIcon(QIcon(":/resources/max.png"));
    m_buttonMinimize->setIcon(QIcon(":/resources/min.png"));
    m_buttonRestore->setIcon(QIcon(":/resources/restore.png"));

    // 设置按钮的最大宽度，例如限制为 30 像素
    m_buttonClose->setMaximumWidth(30);
    m_buttonMaximize->setMaximumWidth(30);
    m_buttonMinimize->setMaximumWidth(30);
    m_buttonRestore->setMaximumWidth(30);



    connect(m_buttonClose, &QAbstractButton::clicked, this, &SystemButtonBar::onButtonCloseClicked);
    connect(m_buttonMaximize, &QAbstractButton::clicked, this, &SystemButtonBar::onButtonMaximizeClicked);
    connect(m_buttonMinimize, &QAbstractButton::clicked, this, &SystemButtonBar::onButtonMinimizeClicked);
    connect(m_buttonRestore, &QAbstractButton::clicked, this, &SystemButtonBar::onButtonRestoreClicked);

    m_mainLayout->addWidget(m_buttonMinimize, 0, Qt::AlignTop);
    m_mainLayout->addWidget(m_buttonMaximize, 0, Qt::AlignTop);
    m_mainLayout->addWidget(m_buttonRestore, 0, Qt::AlignTop);
    m_mainLayout->addWidget(m_buttonClose, 0, Qt::AlignTop);

    m_buttonRestore->hide();

    this->setLayout(m_mainLayout);
}

void SystemButtonBar::setMaximized()
{
    m_buttonMaximize->hide();
    m_buttonRestore->show();
}

void SystemButtonBar::setRestored()
{
    m_buttonMaximize->show();
    m_buttonRestore->hide();
}

void SystemButtonBar::installOn(QWidget *widget)
{
    m_widgetInstalledOn = widget;
}


void SystemButtonBar::setMinimizeButtonEnabled(bool enabled)
{
    if (enabled)
    {
        if (m_buttonMinimize->parent() == 0)
            m_mainLayout->insertWidget(0, m_buttonMinimize, 0, Qt::AlignTop);
    }
    else
    {
        m_mainLayout->removeWidget(m_buttonMinimize);
        m_buttonMinimize->setParent(0);
    }
}

void SystemButtonBar::setMaximizeButtonEnabled(bool enabled)
{
    if (enabled )
    {
        if (m_buttonMaximize->parent() == 0)
        {
            if (m_buttonClose->parent() != 0)
            {
                m_mainLayout->insertWidget(m_mainLayout->count() - 1, m_buttonMaximize, 0, Qt::AlignTop);
                m_mainLayout->insertWidget(m_mainLayout->count() - 1, m_buttonRestore, 0, Qt::AlignTop);
            }
            else
            {
                m_mainLayout->addWidget(m_buttonMaximize, 0, Qt::AlignTop);
                m_mainLayout->addWidget(m_buttonRestore, 0, Qt::AlignTop);
            }
        }
    }
    else
    {
        m_mainLayout->removeWidget(m_buttonMaximize);
        m_buttonMaximize->setParent(0);
        m_mainLayout->removeWidget(m_buttonRestore);
        m_buttonRestore->setParent(0);
    }
}

void SystemButtonBar::setCloseButtonEnabled(bool enabled)
{
    if (enabled)
    {
        if (m_buttonClose->parent() == 0)
            m_mainLayout->addWidget(m_buttonClose, 0, Qt::AlignTop);
    }
    else
    {
        m_mainLayout->removeWidget(m_buttonClose);
        m_buttonClose->setParent(0);
    }
}

QString SystemButtonBar::getTheme()
{
    return "qwe";
}

void SystemButtonBar::onThemeChanged(const QString &theme)
{
//    if (theme == PublicConstants::THEME_BLUE_NAME)
//    {
//        m_buttonClose->setImageName("://style/blue/close");
//        m_buttonMaximize->setImageName("://style/blue/max");
//        m_buttonMinimize->setImageName("://style/blue/min");
//        m_buttonRestore->setImageName("://style/blue/restore");
//    }
//    else if (theme == PublicConstants::THEME_BLACK_NAME)
//    {
//        m_buttonClose->setImageName("://style/black/close");
//        m_buttonMaximize->setImageName("://style/black/max");
//        m_buttonMinimize->setImageName("://style/black/min");
//        m_buttonRestore->setImageName("://style/black/restore");
//    }
}

void SystemButtonBar::onButtonMinimizeClicked()
{
    if (m_widgetInstalledOn)
    {
        m_widgetInstalledOn->showMinimized();
    }
}

void SystemButtonBar::onButtonMaximizeClicked()
{
    if (m_widgetInstalledOn)
    {
        setMaximized();
        m_widgetInstalledOn->showMaximized();
    }
}

void SystemButtonBar::onButtonRestoreClicked()
{
    if (m_widgetInstalledOn)
    {
        setRestored();
        m_widgetInstalledOn->showNormal();
    }
}

void SystemButtonBar::onButtonCloseClicked()
{
    if (m_widgetInstalledOn)
    {
        m_widgetInstalledOn->close();
    }
}
