﻿#include "TitleWidget.h"

#include "UiFrame.h"
#include "SystemButtonBar.h"

TitleWidget::TitleWidget(UiFrame *uiFrame, QWidget *parent)
    : QWidget(parent)
    , m_uiFrame(uiFrame)
    , m_mousePressed(false)
{
    setupUi();
}

void TitleWidget::setupUi()
{
    setAttribute(Qt::WA_StyledBackground, true);

    setFixedHeight(30);

    QHBoxLayout *titleLayout = new QHBoxLayout;
    titleLayout->setSpacing(10);
    titleLayout->setContentsMargins(10, 0, 2, 0);
    setLayout(titleLayout);


    // TitleTest
    {
        m_titleLabel = new QLabel;
        titleLayout->addWidget(m_titleLabel);
        m_titleLabel->setAlignment(Qt::AlignCenter);
    }

    titleLayout->addStretch();

    // SystemButtonBar
    {
        m_systemButtonBar = new SystemButtonBar;
        m_systemButtonBar->installOn(m_uiFrame);
        titleLayout->addWidget(m_systemButtonBar);
    }
}

void TitleWidget::setupConnects()
{
    connect(this, SIGNAL(themeChanged(const QString&)), m_systemButtonBar, SLOT(onThemeChanged(const QString&)));
}

void TitleWidget::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton)
    {
        if (event->y() < 10 || event->x() < 10
                || rect().width()- event->x() < 10)
        {
            m_mousePressed = false;
            event->ignore();
            return;
        }

        m_mousePressed = true;
        m_pressPoint = event->globalPos();
    }

    event->ignore();
}

void TitleWidget::mouseReleaseEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton)
    {
        m_mousePressed = false;
    }

    event->ignore();
}

void TitleWidget::mouseDoubleClickEvent(QMouseEvent *event)
{
    if (m_uiFrame->isMaximized())
    {
        m_uiFrame->showNormal();
        m_systemButtonBar->setRestored();
    }
    else
    {
        m_uiFrame->showMaximized();
        m_systemButtonBar->setMaximized();
    }

    event->ignore();
}

void TitleWidget::mouseMoveEvent(QMouseEvent *event)
{
    if (m_mousePressed)
    {
        m_movePoint = event->globalPos();

        if (!m_uiFrame->isMaximized())
            m_uiFrame->move(m_uiFrame->pos() + m_movePoint - m_pressPoint);

        m_pressPoint = m_movePoint;
    }

    event->ignore();
}

void TitleWidget::setLogo(const QPixmap &pixmap, const QSize &size)
{
    m_logoLabel->setFixedSize(size);
    m_logoLabel->setPixmap(pixmap);
    m_logoLabel->setScaledContents(true);
}

void TitleWidget::setTitleText(const QString &titleText)
{
    m_titleLabel->setText(titleText);
}
