#include "Ribbon.h"
#include "TitleWidget.h"
#include "UiFrame.h"

#include <QDockWidget>
#include <QMenuBar>
#include <QToolBar>
#include <QToolButton>
#include <QTreeWidget>
#include <QDebug>
#include <QTextEdit>
#include <QMdiArea>
#include <QSettings>
#include <DockWidgetTab.h>
#include <QSplitter>
//#include <FramelessHelper.h>
#include <plugins/CA_ProjectManager/ProjectManagerWidget.h>

#include <utils/ServiceRegistry.h>

UiFrame::UiFrame()
{
    setupUi();
    ServiceRegistry<UiFrameService>::instance().registerService(this);
    //    FramelessHelper *tmp = new FramelessHelper(this);

    //    tmp->activateOn(this);
    //    tmp->setWidgetMovable(false);
    //    tmp->setWidgetResizable(true);
    //    tmp->useRubberBandOnResize(false);
}

UiFrame::~UiFrame()
{
    QByteArray state = m_dockManager->saveState();
    QSettings settings("config/file.ini", QSettings::IniFormat);
    settings.setValue("stateData", QVariant(state));
    settings.setValue("windowSize", size());
    m_dockManager->savePerspectives(settings);
}

void UiFrame::setupUi()
{
    //    setWindowFlags(Qt::FramelessWindowHint);
    //    setMinimumSize(1280, 720);

    m_mainLayout = new QVBoxLayout;
    m_mainLayout->setSpacing(0);
    m_mainLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(m_mainLayout);

    m_mainWindow = new QMainWindow;
    m_mainWindow->setStyleSheet("font-family: Microsoft YaHei, 微软雅黑;");
    m_mainWindow->setStyleSheet("background:white;");
    setWindowTitle(QStringLiteral("深部工程软件"));
    QWidget *ribbonDockWidget = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout(ribbonDockWidget);
    layout->setContentsMargins(0,0,0,0);

    m_ribbonTabWidget = new Ribbon;
    layout->addWidget(m_ribbonTabWidget);
    ribbonDockWidget->setStyleSheet("background-color:rgb(234,237,238);font-family: Microsoft YaHei, 微软雅黑;");
    ribbonDockWidget->setFixedHeight(100);

    m_mainLayout->addWidget(ribbonDockWidget);


    m_ribbonTabWidget->addTab(QStringLiteral(""));
    m_ribbonTabWidget->addTab(QStringLiteral("前处理"));
    m_ribbonTabWidget->addTab(QStringLiteral("材料"));
    m_ribbonTabWidget->addTab(QStringLiteral("物理场"));
    m_ribbonTabWidget->addTab(QStringLiteral("求解器"));
    m_ribbonTabWidget->addTab(QStringLiteral("结果"));
    m_ribbonTabWidget->addTab(QStringLiteral(""));

    //------------------------------------------------------------------------------
//    QToolButton *m_viewButton = new QToolButton;
//    m_viewButton->setText(QStringLiteral("视图"));

//    m_ribbonTabWidget->setTabEnabled(6, false);
//    m_ribbonTabWidget->tabBar()->setTabButton(6, QTabBar::RightSide, m_viewButton);

//    QString buttonstyle = "QPushButton {"
//                   "    background-color: rgb(54, 140, 203); /* 设置背景色 */"
//                    "  text-align: center;"
//                    "padding:0px;"
//                    "padding-left:5px;"
//                    "font-size:17px;"
//                    "font-family: Microsoft YaHei, 微软雅黑;"
//                   "    color: white; /* 设置文本颜色 */"
//                   "}"
//                    "QPushButton::menu-indicator{image:none}"
//                   "QPushButton:hover {"
//                   "    background-color: #0078D7; /* 设置鼠标悬停时的背景色 */"
//                   "    color: black; /* 设置鼠标悬停时的文本颜色 */"
//                   "}"
//                   "QPushButton:pressed {"
//                   "    background-color: #005499; /* 设置按钮按下时的背景色 */"
//                   "}";

//    m_viewButton->setStyleSheet(buttonstyle);

//    QMenu *menu = new QMenu(m_viewButton);

//    QString style = "QMenu {"
//                   "    background-color: #F0F0F0; /* 设置背景色 */"
//                   "    border: 1px solid #707070; /* 设置边框 */"
//                   "}"

//                   "QMenu::item {"
//                   "    padding: 5px 20px; /* 设置每个菜单项的内边距 */"
//                   "}"
//                   "QMenu::item:selected {"
//                   "    background-color: #0078D7; /* 设置选中时的背景色 */"
//                   "    color: white; /* 设置选中时的文本颜色 */"
//                   "}";

//    menu->setStyleSheet(style);

//    // 创建 QActions，并添加到 QMenu
//    QAction *m_newProjectAction = new QAction(QStringLiteral("模型开发器"), menu);
//    m_openProjectAction = new QAction(QStringLiteral("设置"), menu);
//    m_saveProjectAction = new QAction(QStringLiteral("图形"), menu);
//    m_reSaveProjectAction = new QAction(QStringLiteral("另存为"), menu);
//    m_helpAction = new QAction(QStringLiteral("帮助"), menu);
//    menu->addAction(m_newProjectAction);
//    menu->addAction(m_openProjectAction);
//    menu->addAction(m_saveProjectAction);
//    menu->addAction(m_reSaveProjectAction);
//    menu->addAction(m_helpAction);


    //-------------------------------------------------------

    m_mainLayout->addWidget(m_mainWindow);


    ads::CDockManager::setConfigFlag(ads::CDockManager::DockAreaHasCloseButton, false);

    ads::CDockManager::setConfigFlag(ads::CDockManager::ActiveTabHasCloseButton, false);
    ads::CDockManager::setConfigFlag(ads::CDockManager::DockAreaHasTabsMenuButton, true);
    ads::CDockManager::setConfigFlag(ads::CDockManager::DockAreaHasUndockButton, false);

    ads::CDockManager::setConfigFlag(ads::CDockManager::OpaqueSplitterResize, true);
    ads::CDockManager::setConfigFlag(ads::CDockManager::XmlCompressionEnabled, false);
    ads::CDockManager::setConfigFlag(ads::CDockManager::FocusHighlighting, true);
    ads::CDockManager::setAutoHideConfigFlags(ads::CDockManager::DefaultAutoHideConfig);

    m_dockManager = new ads::CDockManager(m_mainWindow);
    QString qss = R"(
                  QSplitter::handle {
                  height: 4px; /* 间接调整分隔线的高度（如果是水平分隔线） */
                     width: 4px; /* 间接调整分隔线的宽度（如果是垂直分隔线） */
                  }
                  ads--CDockAreaWidget[focused="true"]  {
                  background: white;

                  border-color: rgb(96,181,247);
                  }

                  #dockAreaAutoHideButton {
                      qproperty-icon: url(:/resources/images/vs-pin-button.svg),
                          url(:/resources/images/vs-pin-button-disabled.svg) disabled;
                      qproperty-iconSize: 16px;
                  }

                  ads--CAutoHideDockContainer #dockAreaAutoHideButton {
                      qproperty-icon: url(:/resources/images/vs-pin-button-pinned-focused.svg);
                      qproperty-iconSize: 16px;
                  }


                  ads--CDockAreaWidget #tabsMenuButton::menu-indicator {
                      image: none;
                  }
                  QScrollArea#dockWidgetScrollArea {
                  padding: 0px;
                  }
                  QScrollArea#dockWidgetScrollArea {
                      border: 0px solid rgb(200, 200, 200);
                  }
                  ads--CDockAreaWidget {
                      border: 1px solid rgb(200, 200, 200);
                  }
                  ads--CDockWidgetTab  {
                      background: red;
                  }
                  QLabel#dockWidgetTabLabel {
                      text-align: left;
                  }
                  QScrollBar:vertical {
                      width: 10px; /* 设置垂直滚动条的宽度 */
                      background: #f0f0f0; /* 设置滚动条的背景颜色 */
                      border: none; /* 取消垂直滚动条的边框样式 */
                      border-radius: 5px; /* 设置滚动条的圆角 */
                  }

                  QScrollBar:horizontal {
                      height: 10px; /* 设置水平滚动条的高度 */
                      background: #f0f0f0; /* 设置滚动条的背景颜色 */
                      border: none; /* 取消水平滚动条的边框样式 */
                      border-radius: 5px; /* 设置滚动条的圆角 */
                  }

                  QScrollBar::handle:vertical {
                      background: #c0c0c0; /* 设置垂直滚动条滑块的背景颜色 */
                  }

                  QScrollBar::handle:horizontal {
                      background: #c0c0c0; /* 设置水平滚动条滑块的背景颜色 */
                  }
                  )";
    m_dockManager->setStyleSheet(qss);

    {
        QWidget* initWidget = new QWidget();
        m_initDockWidget = new ads::CDockWidget(QString("123"));
        m_initDockWidget->setWidget(initWidget);
        m_dockManager->setCentralWidget(m_initDockWidget);
        m_dockManager->addDockWidget(ads::CenterDockWidgetArea, m_initDockWidget);
        m_initDockWidget->setFeature(ads::CDockWidget::DockWidgetFeature::DockWidgetMovable, false);
        m_initDockWidget->setFeature(ads::CDockWidget::DockWidgetFeature::DockWidgetClosable, true);
        m_initDockWidget->setFeature(ads::CDockWidget::DockWidgetFeature::DockWidgetFloatable, false);
        m_initDockWidget->toggleView(true);
        //        initWidget->setVisible(false);
    }
}

void UiFrame::addWidget(QWidget *widget, QString name)
{
    ads::CDockWidget *dockWidget = new ads::CDockWidget(name);
//    dockWidget->setFeature(ads::CDockWidget::DockWidgetFeature::NoTab,true);
    dockWidget->setWidget(widget);

    // 将dockWidget添加到主页面
    m_dockManager->addDockWidget(ads::TopDockWidgetArea, dockWidget, m_initDockWidget->dockAreaWidget());
}

void UiFrame::setSettingWidget(QWidget *widget)
{
    m_settingDockWidget->setWidget(widget);
}

void UiFrame::restoreState()
{
    QSettings settings("config/file.ini", QSettings::IniFormat);
    QByteArray savedState = settings.value("stateData").toByteArray();
    // 使用 dockManager->restoreState() 恢复状态
    bool success = m_dockManager->restoreState(savedState);

    if (settings.contains("windowSize")) {
        resize(settings.value("windowSize").toSize());
    }

    if (!success) {
        // 恢复状态失败的处理逻辑
    }
}

void UiFrame::addMenuButton(QString tabName, QString groupName, QToolButton *button)
{
    m_ribbonTabWidget->addButton(tabName, groupName, button);
}

void UiFrame::addFileButton(QPushButton *button)
{
    m_ribbonTabWidget->addFileButton(button);
}
