﻿#ifndef VISUALWIDGET_H
#define VISUALWIDGET_H

#include <QWidget>
#include <vtkActor2D.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <QDebug>
#include <public/services/PreProcessingService.h>
#include <public/services/UiFrameService.h>
#include <public/services/VisualWidgetService.h>
#include <vtkSmartPointer.h>
#include <vtkUnstructuredGrid.h>
#include <vtkSmartPointer.h>
#include <vtkXMLUnstructuredGridReader.h>
#include <vtkUnstructuredGrid.h>
#include <vtkDataSetMapper.h>
#include <vtkQuad.h>
#include <vtkStructuredGrid.h>
//#include <vtkActor.h>
#include <vtkRenderWindow.h>
#include <vtkRenderer.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkLookupTable.h>
#include <vtkPointData.h>
#include <QDebug>
#include <vtkAutoInit.h>
#include <vtkFloatArray.h>
#include <vtkDoubleArray.h>
#include <vtkContourFilter.h>
#include <vtkPolyDataMapper.h>
#include <vtkCutter.h>
#include <vtkPlane.h>
#include <vtkLineSource.h>
#include <vtkProbeFilter.h>
#include <vtkTable.h>
#include <vtkIntArray.h>
#include <vtkChartXY.h>
#include <vtkContextView.h>
#include <vtkContextScene.h>
#include <vtkPlot.h>
#include <vtkAxesActor.h>
#include <vtkArrowSource.h>
#include <vtkGlyph3DMapper.h>
#include <vtkPointSource.h>
#include <vtkStreamTracer.h>
#include <vtkRungeKutta4.h>
#include <vtkArrayCalculator.h>
#include <vtkStreamTracer.h>
#include <vtkProperty.h>
#include <QThread>
#include <vtkOrientationMarkerWidget.h>
#include <vtkCaptionActor2D.h>
#include <vtkTextProperty.h>
#include "vtkSmartPointer.h"
#include "vtkMultiBlockDataSet.h"
#include "vtkPointData.h"
#include "vtkCellData.h"
#include "vtkDataSetMapper.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"
#include "vtkRenderWindow.h"
#include "vtkRenderer.h"
#include "vtkCellDataToPointData.h"
#include "vtkAppendFilter.h"
#include <QInputDialog>
#include <QDir>
#include <QString>
#include <vtkCellPicker.h>
#include <vtkRendererCollection.h>
#include <utils/ServiceRegistry.h>
#include <vtkSphereSource.h>
#include <vtkTextActor.h>

namespace Ui {
class VisualWidget;
}

class VisualWidget : public QWidget,public VisualWidgetService
{
    Q_OBJECT

public:
    explicit VisualWidget(QWidget *parent = nullptr);
    ~VisualWidget();

    //    void nephogramTypeIndexChanged(int i);

    void addActor(vtkSmartPointer<vtkActor> actor);
    void addActor2D(vtkSmartPointer<vtkActor2D> actor);
    void removeAllActor();


    void reFreash();
    void removeGridActor(vtkSmartPointer<vtkActor> actor);

    vtkSmartPointer<vtkRenderWindow> getrenderWindow();
protected:
    void showEvent(QShowEvent *event) override
    {
        // 首次调用 show() 函数时执行的逻辑
        qDebug()<<"update VisualWidget";
        m_renderWindow->Render();

        // 调用父类的 showEvent 函数，确保正常的事件处理
        QWidget::showEvent(event);
    }
private:
    void initVTKWidget();

private:
    Ui::VisualWidget *ui;
    UiFrameService *m_uiFrameService;

    vtkSmartPointer<vtkRenderer> m_renderer;
    vtkSmartPointer<vtkRenderWindow> m_renderWindow;
    vtkSmartPointer<vtkRenderWindowInteractor> m_renderWindowInteractor;
    vtkSmartPointer<vtkActor> m_mainActor;
    QVector<vtkSmartPointer<vtkActor> > m_tmpActor;
    vtkSmartPointer<vtkActor2D> m_scalarBar;
    vtkSmartPointer<vtkActor> m_gridActor;

    void setupButton();
    vtkSmartPointer<vtkRenderer> getRender();
    vtkSmartPointer<vtkRenderWindowInteractor> getInteractor();
    void removeActor(vtkSmartPointer<vtkActor> actor);
};

#endif // VISUALWIDGET_H
